#========================================================================================
# package      DE_EPAGES::Amazon::t::RemoteMWSToMerchantTest
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for RemoteMWSToMerchant adapter using mocks
#========================================================================================
package DE_EPAGES::Amazon::t::RemoteMerchantToMWSAdapterTest;
use base Test::Class;

use DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_PENDING_FEEDS );
use Test::More;
use Test::MockObject;
use HTTP::Response;
use strict;

# setup methods are run before every test method.
sub setUp : Test(setup) {
    my $self = shift;

    $self->{MWSMock}  = Test::MockObject->new();
    $self->{HTTPMock} = Test::MockObject->new();
    $self->{Remote}   = DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter->new(
        AWSURL         => 1,
        AWSAccessKeyId => 2,
        AWSSecretKeyId => 3,
        MerchantID     => 4,
        MarketplaceID  => 5,
        mws            => $self->{MWSMock}
    );
}

sub testCredentials : Test(5) {
    my $self = shift;
    is( $self->{Remote}->url,         1, 'AWSURL' );
    is( $self->{Remote}->marketplace, 5, 'marketplace' );
    is( $self->{Remote}->merchant,    4, 'merchant' );
    ok( !$self->{Remote}->is_logging, 'no logging' );
    ok( $self->{Remote}->mws,         'mws adapter existing' );
}

sub testLogin : Test {
    my $self = shift;
    $self->{MWSMock}->mock(
        'getOrderServiceStatus',
        sub {
            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->login();
    $self->{MWSMock}->called_ok('getOrderServiceStatus');
}

sub testActivateOrderScheduler : Test(3) {
    my $self     = shift;
    my $Interval = '_1_HOUR_';

    $self->{MWSMock}->mock(
        'manageReportSchedule',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'ReportType'}, $AMAZON_PENDING_FEEDS{'orders'}, 'ReportType' );
            is( $Info{'Schedule'}, $Interval, 'Interval' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->activateOrderScheduler($Interval);
    $self->{MWSMock}->called_ok('manageReportSchedule');
}

sub testGetAllPendingDocumentInfo : Test(4) {
    my $self = shift;
    $self->_prepareMWSMockForGetAllPendingDocumentInfo();

    my $Ret = $self->{Remote}->getAllPendingDocumentInfo();
    $self->{MWSMock}->called_ok('getReportList');
    $self->{MWSMock}->called_ok('getReportListByNextToken');
    is_deeply( $Ret->Result(), [ '898899473', '898899474' ], 'report ids.' );
}

sub _prepareMWSMockForGetAllPendingDocumentInfo {
    my $self = shift;
    $self->{MWSMock}->mock(
        'getReportList',
        sub {
            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0"?>
<GetReportListResponse xmlns="http://mws.amazonaws.com/doc/2009-01-01/">
  <GetReportListResult>
  <NextToken>2YgYW55IPQhvu5hbCBwbGVhc3VyZS4=</NextToken>
    <HasNext>true</HasNext>
    <ReportInfo>
      <ReportId>898899473</ReportId>
      <ReportType>_GET_MERCHANT_LISTINGS_DATA_</ReportType>
      <ReportRequestId>2278662938</ReportRequestId>
      <AvailableDate>2009-02-10T09:22:33+00:00</AvailableDate>
      <Acknowledged>false</Acknowledged>
    </ReportInfo>
  </GetReportListResult>
  <ResponseMetadata>
    <RequestId>fbf677c1-dcee-4110-bc88-2ba3702e331b</RequestId>
  </ResponseMetadata>
</GetReportListResponse>
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );

    $self->{MWSMock}->mock(
        'getReportListByNextToken',
        sub {
            my $self = shift;
            my (%Info) = @_;
            is( $Info{'NextToken'}, '2YgYW55IPQhvu5hbCBwbGVhc3VyZS4=', 'NextToken' );
            my $Mock = Test::MockObject->new();
            $Mock->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0"?>
<GetReportListByNextTokenResponse xmlns="http://mws.amazonaws.com/doc/2009-01-01/">
<GetReportListByNextTokenResult>
  <NextToken>none</NextToken>
  <HasNext>false</HasNext>
  <ReportInfo>
    <ReportId>898899474</ReportId>
    <ReportType>_GET_MERCHANT_LISTINGS_DATA_</ReportType>
    <ReportRequestId>2278662938</ReportRequestId>
    <AvailableDate>2009-02-10T09:22:33+00:00</AvailableDate>
    <Acknowledged>false</Acknowledged>
  </ReportInfo>
  </GetReportListByNextTokenResult>
<ResponseMetadata>
  <RequestId>fbf677c1-dcee-4110-bc88-2ba3702e331b</RequestId>
</ResponseMetadata>
</GetReportListByNextTokenResponse>
EOL
                }
            );
        }
    );
}

sub testGetDocumentProcessingStatus : Test(2) {
    my $self  = shift;
    my $DocId = 'DOCUMENT_1';
    $self->{MWSMock}->mock(
        'getFeedSubmissionList',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'FeedSubmissionIdList.Id.1'}, $DocId, 'document id' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->getDocumentProcessingStatus($DocId);
    $self->{MWSMock}->called_ok('getFeedSubmissionList');
}

sub testGetProcessingReportDocument : Test(2) {
    my $self  = shift;
    my $DocId = 'DOCUMENT_1';
    $self->{MWSMock}->mock(
        'getFeedSubmissionResult',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'FeedSubmissionId'}, $DocId, 'document id' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->getProcessingReportDocument($DocId);
    $self->{MWSMock}->called_ok('getFeedSubmissionResult');
}

sub testGetOrderDocument : Test(2) {
    my $self  = shift;
    my $DocId = 'DOCUMENT_1';
    $self->{MWSMock}->mock(
        'getReport',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'ReportId'}, $DocId, 'document id' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->getOrderDocument($DocId);
    $self->{MWSMock}->called_ok('getReport');
}

sub tesstPostDocument : Test(3) {
    my $self     = shift;
    my $FeedType = '_TEST_';
    $self->{MWSMock}->mock(
        'submitFeed',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'FeedType'},        $FeedType, 'feedType' );
            is( ref( $Info{'rContent'} ), 'REF',     'content' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0"?>
<SubmitFeedResponse xmlns="http://mws.amazonaws.com/doc/2009-01-01/"><SubmitFeedResult><FeedSubmissionInfo><FeedSubmissionId>2533151448</FeedSubmissionId><FeedType>_POST_OFFER_ONLY_DATA_</FeedType><SubmittedDate>2012-01-03T10:39:10+00:00</SubmittedDate><FeedProcessingStatus>_SUBMITTED_</FeedProcessingStatus></FeedSubmissionInfo></SubmitFeedResult><ResponseMetadata><RequestId>7a432b03-df21-4870-9f5e-bbdad8a1a78e</RequestId></ResponseMetadata></SubmitFeedResponse>
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );
    $self->{Remote}->postDocument( $FeedType, \"TestData" );
    $self->{MWSMock}->called_ok('submitFeed');
}

sub testPostDocumentDownloadAck : Test(3) {
    my $self = shift;
    $self->{MWSMock}->mock(
        'updateReportAcknowledgements',
        sub {
            shift;
            my (%Info) = @_;
            is( $Info{'ReportIdList.Id.1'}, '1234',  'doc1' );
            is( $Info{'ReportIdList.Id.2'}, '56789', 'doc2' );

            $self->{HTTPMock}->mock(
                'decoded_content',
                sub {
                    return << 'EOL';
<?xml version="1.0" ?>
<root />
EOL
                }
            );
            return $self->{HTTPMock};
        }
    );

    $self->{Remote}->postDocumentDownloadAck( [qw( 1234 56789 )] );
    $self->{MWSMock}->called_ok('updateReportAcknowledgements');
}

package main;    ## no critic (ProhibitMultiplePackages)
use DE_EPAGES::Core::API::Script qw ( RunScript );

RunScript(
    Sub => sub {
        Test::Class->runtests;
    }
);

1;
