use strict;

use DE_EPAGES::Amazon::API::AmazonTransaction;
use DE_EPAGES::Amazon::API::DTO::Address;
use DE_EPAGES::Amazon::API::DTO::Component;
use DE_EPAGES::Amazon::API::DTO::Item;
use DE_EPAGES::Amazon::API::DTO::OrderReport;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp UsePackage );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadClassByAlias
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS TAX_MODEL_NET );
use DE_EPAGES::Amazon::API::SOAP::GetOrderDocumentResponseHandler;
use DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler;

use SOAP::Lite;
use Test::More;
use File::Basename;
my $ROOTPATH = '/Test';

#  DE_EPAGES::Amazon::API::SOAP::GetOrderDocumentResponseHandler or
#  DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler
my $HANDLERPACKAGE = 'DE_EPAGES::Amazon::API::MWS::GetOrderReportResponseHandler';

## no critic (ProhibitMagicNumbers)

####### test data ############
sub _getTestDataGermany {    ## no critic (ProhibitFunctionSize)
    my $O = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
    $O->AmazonOrderID('302-6636987-0853930');
    $O->AmazonSessionID('302-6636987-0853930');
    $O->OrderDate('2009-06-30T13:17:49+00:00');
    $O->ReportNumber('922629424');

    $O->BuyerEmailAddress('buyer@example.com');
    $O->BuyerName('Hansi Müller');

    $O->Currency('EUR');
    $O->FulfillmentAddress('Lieferstr. 553');
    $O->FulfillmentMethod('Ship');
    $O->FulfillmentServiceLevel('Standard');

    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('Kaeufergasse 123');
    $B->AddressFieldTwo();
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    $O->BuyerAddress($B);

    my $F = DE_EPAGES::Amazon::API::DTO::Address->new;
    $F->Name('Petra Zwegert');
    $F->AddressFieldOne('Lieferstr. 14');
    $F->AddressFieldTwo('c/o Hermine Meier');
    $F->City('Hamburg');
    $F->StateOrRegion('Hamburg');
    $F->PostalCode('20345');
    $F->CountryCode('DE');
    $F->PhoneNumber('+49 40 123 123433');

    $O->FulfillmentAddress($F);

    my $I = DE_EPAGES::Amazon::API::DTO::Item->new;
    $I->AmazonOrderItemCode('05991383163091');
    $I->ProductTaxCode('PTC_PRODUCT_TAXABLE_A');
    $I->Title('Campingaz CV470 Ventilgaskartusche');
    $I->Quantity('2');
    $I->SKU('cg_0101104470');

    my $P = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P->Amount(1.55);
    $P->Type('Principal');

    my $P2 = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P2->Amount(3.55);
    $P2->Type('Shipping');

    $I->ItemPriceArray($P);
    $I->ItemPriceArray($P2);

    $O->ItemArray($I);

    return [$O];
}

sub _getTestDataSwitzerland {    ## no critic (ProhibitFunctionSize)
    my $O = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
    $O->AmazonOrderID('302-6636987-0853931');
    $O->AmazonSessionID('302-6636987-0853931');
    $O->OrderDate('2009-06-30T13:17:50+00:00');
    $O->ReportNumber('922629425');

    $O->BuyerEmailAddress('buyer@example.ch');
    $O->BuyerName('Albert Schweitzer');

    $O->Currency('EUR');
    $O->FulfillmentAddress('Lieferstr. 553');
    $O->FulfillmentMethod('Ship');
    $O->FulfillmentServiceLevel('Standard');

    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Albert Schweitzer');
    $B->AddressFieldOne('Kaeufergasse 123');
    $B->AddressFieldTwo();
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    $O->BuyerAddress($B);

    my $F = DE_EPAGES::Amazon::API::DTO::Address->new;
    $F->Name('Albert Schweitzer');
    $F->AddressFieldOne('Lieferstr. 14');
    $F->AddressFieldTwo('c/o Hermine Meier');
    $F->City('Genf');
    $F->StateOrRegion('Genf');
    $F->PostalCode('4321');
    $F->CountryCode('CH');
    $F->PhoneNumber('+41 61 123456');

    $O->FulfillmentAddress($F);

    my $I = DE_EPAGES::Amazon::API::DTO::Item->new;
    $I->AmazonOrderItemCode('05991383163091');
    $I->ProductTaxCode('PTC_PRODUCT_TAXABLE_A');
    $I->Title('Campingaz CV470 Ventilgaskartusche');
    $I->Quantity('2');
    $I->SKU('cg_0101104470');

    my $P = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P->Amount(1.55);
    $P->Type('Principal');

    my $P2 = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P2->Amount(3.55);
    $P2->Type('Shipping');

    $I->ItemPriceArray($P);
    $I->ItemPriceArray($P2);

    $O->ItemArray($I);

    return [$O];
}

sub _getTestDataFromXML {
    my ($filename) = @_;
    my $Parser     = SOAP::Deserializer->new;
    my $SOM        = $Parser->deserialize( ${ GetFileContent($filename) } );
    UsePackage($HANDLERPACKAGE);
    my $Handler    = $HANDLERPACKAGE->new();
    $Handler->identifier('12345');

    # we are testing the internal handler method.
    $Handler->_handleOrderReport($SOM);    ## no critic (ProhibitPrivateSubroutineUse)
    my $aTree = $Handler->Result();
    return $aTree;
}
####### /test data ############

####### test helper ############
sub _testOrderImport {
    my ( $cTestData, $hRefValues ) = @_;
    my $Shop      = LoadObjectByPath($ROOTPATH)->child('TestShopAmazon');
    my $Merchant  = $Shop->folder('PortalConfigs')->child('AmazonTestConfig');
    my $aTestData = $cTestData->();

    foreach my $testData (@$aTestData) {

        # prepare stock test
        my $Product  = $Shop->folder('Products')->child('cg_0101104470');
        my $OldStock = $Product->get('StockLevel');

        my $AmazonTransaction = DE_EPAGES::Amazon::API::AmazonTransaction->new;
        my $cInsertTransSub   = sub {
            $AmazonTransaction->insert( $Merchant, [$testData] );
        };

        if ( !$hRefValues->{'StockReduce'} ) {
            BlockWarnings( $cInsertTransSub, [qw(NoProductForSKU)] );
        }
        else {
            &$cInsertTransSub;
        }

        my $hRefOrderValues = $hRefValues->{ $testData->{AmazonOrderID} };
        my $TestName        = $hRefOrderValues->{'TestName'};
        my $dbtlist
          = LoadClassByAlias('AmazonTransaction')->getObjects( 'ByExternalOrderID', [ $testData->{AmazonOrderID} ] );
        is( scalar @$dbtlist, 1, $TestName . '1 transaction generated' );

        # search order..
        my $matchingLineItems = $AmazonTransaction->getImportedLineItems();
        my $lineItem          = pop @$matchingLineItems;

        # check amounts
        cmp_ok( $lineItem->get('LineItemPrice'), '==', $hRefOrderValues->{'ProductPrice'},
            $TestName . 'product price' );

        my $NewStock;
        $NewStock = $Product->get('StockLevel');
        is(
            $NewStock,
            ( $OldStock - $hRefOrderValues->{'StockReduce'} ),
            $TestName . 'stock before import: ' . $OldStock . ' - after import ' . $NewStock
        );

        my $dbcontainer = $lineItem->get('LineItemContainer');
        my $dborder     = $dbcontainer->parent;
        ok( $dborder->get('PaidOn'),       $TestName . 'order is paid' );
        ok( $dborder->get('AmazonConfig'), $TestName . 'order has AmazonConfig' );

        # check lineitempayment amount
        is( $dbcontainer->get('Payment')->class->alias,
            'LineItemPayment', $TestName . 'Switch to base class LineItemPayment' );

        # check billing address
        my $dbbillingAddress   = $dborder->get('BillingAddress');
        my $testbillingAddress = $testData->BuyerAddress;
        is( $dbbillingAddress->get('City'),    $testbillingAddress->City,            $TestName . 'billing city' );
        is( $dbbillingAddress->get('Street'),  $testbillingAddress->AddressFieldOne, $TestName . 'billing street' );
        is( $dbbillingAddress->get('State'),   $testbillingAddress->StateOrRegion,   $TestName . 'billing state' );
        is( $dbbillingAddress->get('Street2'), $testbillingAddress->AddressFieldTwo, $TestName . 'billing street 2' );
        is( $dbbillingAddress->get('Zipcode'), $testbillingAddress->PostalCode,      $TestName . 'billing plz' );
        is( $dbbillingAddress->get('EMail'),   $testData->BuyerEmailAddress,         $TestName . 'billing email' );

        # check fulfillment address
        my $dbfulfillmentAddress   = $dborder->get('ShippingAddress');
        my $testfulfillmentAddress = $testData->FulfillmentAddress;

        is( $dbfulfillmentAddress->get('City'), $testfulfillmentAddress->City, $TestName . 'shipping city' );
        is(
            $dbfulfillmentAddress->get('Street'),
            $testfulfillmentAddress->AddressFieldOne,
            $TestName . 'shipping street'
        );
        is( $dbfulfillmentAddress->get('State'), $testfulfillmentAddress->StateOrRegion, $TestName . 'shipping state' );
        is(
            $dbfulfillmentAddress->get('Street2'),
            $testfulfillmentAddress->AddressFieldTwo,
            $TestName . 'shipping street 2'
        );
        is( $dbfulfillmentAddress->get('Zipcode'), $testfulfillmentAddress->PostalCode, $TestName . 'shipping plz' );

        # check reference values
        cmp_ok( $dbcontainer->get('GrandTotal'), '==', $hRefOrderValues->{'GrandTotal'}, $TestName . 'grand total' );

        # total tax is a calculated float, so we have to compare using fcmp
        is( fcmp( $dbcontainer->get('TotalTax'), $hRefOrderValues->{'TotalTax'} ), 0, $TestName . 'total tax' )
          or note( 'TotalTax got / expected: '
              . sprintf( '%.16f / %.16f', $dbcontainer->get('TotalTax'), $hRefOrderValues->{'TotalTax'} ) );
        cmp_ok(
            $dbcontainer->get('Shipping')->get('LineItemPrice'),
            '==',
            $hRefOrderValues->{'ShippingPrice'},
            $TestName . 'shipping price'
        );
        is( $dbcontainer->get('TaxArea')->alias, $hRefOrderValues->{'TaxArea'},    $TestName . 'tax area' );
        is( $dbcontainer->get('TaxModel'),       $hRefOrderValues->{'TaxModel'},   $TestName . 'tax model' );
        is( $dbcontainer->get('CurrencyID'),     $hRefOrderValues->{'CurrencyID'}, $TestName . 'currency' );
    }
}
####### /test helper ############

####### tests ############
sub testPromo1 {
    _testOrderImport(
        sub { return _getTestDataFromXML(dirname(__FILE__).'/orderreport_promo1.xml') },
        {
            '028-8438651-3311525' => {
                TestName      => 'PROMO1: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 2.57,
                GrandTotal    => 16.10,
                ProductPrice  => 12.60,
                ShippingPrice => 3.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
            },
        }
    );
}

sub testPromo2 {
    _testOrderImport(
        sub { return _getTestDataFromXML(dirname(__FILE__).'/orderreport_promo2.xml') },
        {
            '028-5256199-7777953' => {
                TestName      => 'PROMO2: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 1.66,
                GrandTotal    => 10.40,
                ProductPrice  => 7.90,
                ShippingPrice => 2.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
            },
        }
    );
}

sub testDataGermany {
    _testOrderImport(
        \&_getTestDataGermany,
        {
            '302-6636987-0853930' => {
                TestName      => 'DE: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                GrandTotal    => 5.10,
                TotalTax      => 0.81,
                ShippingPrice => 3.55,
                ProductPrice  => 1.55,
                StockReduce   => 2,
                CurrencyID    => 'EUR',
            },
        }
    );
}

sub testDataSwitzerland {
    _testOrderImport(
        \&_getTestDataSwitzerland,
        {
            '302-6636987-0853931' => {
                TestName      => 'CH: ',
                TaxArea       => 'non EU',
                TaxModel      => TAX_MODEL_NET,
                GrandTotal    => 5.10,
                TotalTax      => 0.0,
                ShippingPrice => 3.55,
                ProductPrice  => 1.55,
                StockReduce   => 2,
                CurrencyID    => 'EUR',
            },
        }
    );
}

sub testMultiCurrency {
    _testOrderImport(
        sub { return _getTestDataFromXML(dirname(__FILE__).'/orderreport_multicurrency.xml') },
        {
            '326-4366989-9695541' => {
                TestName      => 'MULTI1: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                GrandTotal    => 16.85,
                TotalTax      => 2.69,
                ShippingPrice => 4.50,
                ProductPrice  => 12.35,
                CurrencyID    => 'GBP',
            },
            '128-5256199-7777953' => {
                TestName      => 'MULTI2: ',
                TaxArea       => 'EU',
                TaxModel      => TAX_MODEL_GROSS,
                TotalTax      => 1.66,
                GrandTotal    => 10.40,
                ProductPrice  => 7.90,
                ShippingPrice => 2.50,
                StockReduce   => 0,
                CurrencyID    => 'EUR',
            }
        }
    );
}
####### /tests ############

####### Main ############
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # initialization
            ok( Init(), 'initialization' );

            # tests
            eval {
                # manual data structure 
                testDataGermany();
                testDataSwitzerland();

                # $HANDLERPACKAGE = 'DE_EPAGES::Amazon::API::SOAP::GetOrderDocumentResponseHandler';
                testPromo1();
                testPromo2();
                testMultiCurrency();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'ERROR: ' . $Error->longMessage );
            }

            # cleanup
            ok( Cleanup(), 'cleanup' );
        }
    );
    done_testing;
    return;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    # cleanup AmazonTransactions if they are still in the system.
    foreach my $AmazonOrderID (qw (302-6636987-0853930 302-6636987-0853931 128-5256199-7777953 326-4366989-9695541 )) {
        my $dbtlist = LoadClassByAlias('AmazonTransaction')->getObjects( 'ByExternalOrderID', [$AmazonOrderID] );
        foreach my $dbt ( @{$dbtlist} ) {
            $dbt->delete;
        }
    }

    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( dirname (__FILE__) . '/import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
