use strict;

use DE_EPAGES::Amazon::API::AmazonTransaction;
use DE_EPAGES::Amazon::API::DTO::Address;
use DE_EPAGES::Amazon::API::DTO::Component;
use DE_EPAGES::Amazon::API::DTO::Item;
use DE_EPAGES::Amazon::API::DTO::OrderReport;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadClassByAlias
  LoadObjectByPath
  LoadRootObject
  Transaction
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Order::API::Order qw ( GetEditableOrder );
use DE_EPAGES::Order::API::OrderDocument qw ( CreateOrderDocument );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use Test::More;
use File::Basename;
my $ROOTPATH = '/Test';

sub getTestDataGermany { ## no critic (ProhibitFunctionSize)
    my $O = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
    $O->AmazonOrderID('302-6636987-0853930');
    $O->AmazonSessionID('302-6636987-0853930');
    $O->OrderDate('2009-06-30T13:17:49+00:00');
    $O->ReportNumber('922629424');

    $O->BuyerEmailAddress('buyer@example.com');
    $O->BuyerName('Hansi Müller');

    $O->Currency('EUR');
    $O->FulfillmentAddress('Lieferstr. 553');
    $O->FulfillmentMethod('Ship');
    $O->FulfillmentServiceLevel('Standard');

    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('Kaeufergasse 123');
    $B->AddressFieldTwo();
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    $O->BuyerAddress($B);

    my $F = DE_EPAGES::Amazon::API::DTO::Address->new;
    $F->Name('Petra Zwegert');
    $F->AddressFieldOne('Lieferstr. 14');
    $F->AddressFieldTwo('c/o Hermine Meier');
    $F->City('Hamburg');
    $F->StateOrRegion('Hamburg');
    $F->PostalCode('20345');
    $F->CountryCode('DE');
    $F->PhoneNumber('+49 40 123 123433');

    $O->FulfillmentAddress($F);

    my $I = DE_EPAGES::Amazon::API::DTO::Item->new;
    $I->AmazonOrderItemCode('05991383163091');
    $I->ProductTaxCode('PTC_PRODUCT_TAXABLE_A');
    $I->Title('Campingaz CV470 Ventilgaskartusche');
    $I->Quantity('2');
    $I->SKU('cg_0101104470');

    my $P = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P->Amount(1.55); ## no critic (ProhibitMagicNumbers)
    $P->Type('Principal');

    my $P2 = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P2->Amount(3.55); ## no critic (ProhibitMagicNumbers)
    $P2->Type('Shipping');

    $I->ItemPriceArray($P);
    $I->ItemPriceArray($P2);

    $O->ItemArray($I);

    return $O;
}

sub testOrder2Order { ## no critic (ProhibitFunctionSize)
    my ( $cTestData, $hRefValues ) = @_;
    my $TestName = $hRefValues->{'TestName'};
    my $Shop     = LoadObjectByPath($ROOTPATH)->child('TestShopAmazon');
    my $Merchant = $Shop->folder('PortalConfigs')->child('AmazonTestConfig');

    my $AmazonTransaction = DE_EPAGES::Amazon::API::AmazonTransaction->new;
    my $testData          = $cTestData->();

    my @orderids = $AmazonTransaction->insert( $Merchant, [$testData] );
    ok( scalar @orderids == 1, $TestName . '1 order generated.' );
    my $orderid = pop @orderids;
    my $dbtlist = LoadClassByAlias('AmazonTransaction')->getObjects( 'ByExternalOrderID', [$orderid] );
    is( scalar @{$dbtlist}, 1, $TestName . '1 transaction generated' );

    # search order..
    my $matchingLineItems = $AmazonTransaction->getImportedLineItems();
    is( scalar @{$matchingLineItems}, 1, $TestName . '1 matching lineitem generated' );
    my $lineItem    = pop @{$matchingLineItems};
    my $dbcontainer = $lineItem->get('LineItemContainer');
    my $Order       = $dbcontainer->parent;
    isa_ok( $Order, 'DE_EPAGES::Order::API::Object::CustomerOrder', $TestName . 'Order' );
    my $Document = CreateOrderDocument( $Order, 'Invoice', 'Invoices', {} );
    isa_ok( $Document, 'DE_EPAGES::Order::API::Object::OrderDocument', $TestName . 'new Invoice' );

    my $EditableOrder = GetEditableOrder($Order);

    # clone order if editable order does not yet exist
    unless ( defined $EditableOrder ) {
        Transaction(
            sub {
                $EditableOrder = $Order->clone( {
                        'Parent'        => $Order->get('Customer')->folder('EditableOrders'),
                        'Inherit'       => 1,
                        'OriginalOrder' => $Order
                    },
                    1
                );
            }
        );
    }
    isa_ok( $EditableOrder, 'DE_EPAGES::Order::API::Object::CustomerOrder', $TestName . 'EditableOrder' );
    my $Orders = $Order->parent;

    Transaction(
        sub {
            TriggerHook( 'BeforeEditOrder2Order', { 'Order' => $EditableOrder } );

            # delete the original order and set the parent from the copy to Orders
            $Order->delete;
            $EditableOrder->set( { 'Parent' => $Orders, 'OriginalOrder' => undef } );
        }
    );

    foreach my $LineItem ( @{ $EditableOrder->get('LineItemContainer')->get('LineItems') } ) {
        next unless $LineItem->instanceOf('LineItemAmazonTransaction');
        ok( defined $LineItem->get('AmazonTransaction'), $TestName . 'AmazonTransaction ok.' );
    }

}

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # initialization
            ok( Init(), 'initialization' );

            # tests
            eval { testOrder2Order( \&getTestDataGermany, { TestName => 'O2O: ', } ); };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'ERROR: ' . $Error->longMessage );
            }

            # cleanup
            ok( Cleanup(), 'cleanup' );
        }
    );
    done_testing;
    return;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);

    # cleanup AmazonTransactions if they are still in the system.
    foreach my $AmazonOrderID (qw (302-6636987-0853930 302-6636987-0853931)) {
        my $dbtlist = LoadClassByAlias('AmazonTransaction')->getObjects( 'ByExternalOrderID', [$AmazonOrderID] );
        foreach my $dbt ( @{$dbtlist} ) {
            $dbt->delete;
        }
    }

    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( dirname (__FILE__) . '/import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
