
use strict;
use Test::More tests => 79;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Attributes::IncrementAttribute qw( IncrementIntegerAttribute );
use DE_EPAGES::Object::API::Factory qw( ExistsClassByAlias LoadClassByAlias LoadRootObject );
use DE_EPAGES::Object::API::Table::BooleanAttribute qw( ExistsBooleanAttribute );
use DE_EPAGES::Object::API::Table::FloatAttribute qw( ExistsFloatAttribute );

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';

use constant FLOAT_VALUE => 123.456;
use constant FLOAT_VALUE2 => 0.123456;

sub Main {

    ConnectTestSite();

    # create test classes
    ok( Init(), 'initialization' );

    my $Class1 = LoadClassByAlias( 'C1' );
    ok( $Class1, 'class 1 exists' );

    my $Class3 = LoadClassByAlias( 'C3' );
    ok( $Class3, 'class 3 exists' );

    # single attribute
    my $A1Float = $Class3->attribute( 'A1Float' );
    ok( $A1Float->alias eq 'A1Float', 'attribute name' );
    ok( $A1Float->get('Type') eq 'Float', 'attribute type' );
    ok( $A1Float->get('Package') eq $ATTRIBUTE_PACKAGE, 'attribute package' );
    ok( !defined($A1Float->get('Length')), 'attribute length' );
    ok( !$A1Float->get('IsArray'), 'is not an array' );
    ok( $A1Float->get('IsExportable'), 'is exportable' );
    ok( !$A1Float->get('IsReadOnly'), 'is not read-only' );
    ok( $A1Float->get('AttributeClass')->id == $Class1->id, 'attribute is assigned to class 1' );
    is( $A1Float->get('DefaultValue'), undef, 'no default value specified' );

    my $A2Float = $Class3->attribute( 'A2Float' );
    is( $A2Float->get('DefaultValue'), FLOAT_VALUE, 'default value specified' );

    # all attributes
    my $aAllAttributes1 = $Class1->attributes;
    my $aAllAttributes3 = $Class3->attributes;
    my $Names3 = join '', sort map { $_->alias } @$aAllAttributes3;
    my $AllNames = join( '', sort ('A2Float', 'A3Float', map { $_->alias } @$aAllAttributes1));
    is( $Names3, $AllNames, 'inherited attribute names' );

    my $aDirectAttributes3 = $Class3->directAttributes;
    my $Names3Direct = join '', sort map { $_->alias } @$aDirectAttributes3;
    ok( $Names3Direct eq 'A3Float', 'direct attribute names' );

    ok ($A1Float->get('ExportLevel') eq '2.2', 'export level 2.2');
    ok ($A1Float->get('MajorExportLevel') eq '2', 'major export level 2');

    my $hAttribute2 = $Class3->attribute( 'A2Float' );
    ok (! defined $hAttribute2->get('ExportLevel'), 'no export level');
    ok (! defined $hAttribute2->get('MajorExportLevel'), 'no major export level');

    #+ IncrementIntegerAttribute
    my $IntegerAttribute_0 = $Class1->insertAttribute({
        Alias => 'IntegerAttribute_0',
        Type => 'Integer',
        DefaultValue => 1001,
        Package => $ATTRIBUTE_PACKAGE,
    });

    my $IntegerAttribute_1 = $Class1->insertAttribute({
        Alias => 'IntegerAttribute_1',
        Type => 'Integer',
        Package => $ATTRIBUTE_PACKAGE,
    });

    my $IncIntTest = LoadRootObject->insertChild( $Class1, { Alias => 'IncIntTest' } );
    is( IncrementIntegerAttribute($IncIntTest, $IntegerAttribute_0), '1002', 'IncrementIntegerAttribute + DefaultValue' );
    is( IncrementIntegerAttribute($IncIntTest, $IntegerAttribute_1), '1', 'IncrementIntegerAttribute \ DefaultValue' );
    #-

    # EPG-13530: set attribute without a column
    $Class1->insertAttribute({
        Alias => 'NoColumn',
        Type => 'Float',
        Package => 'DE_EPAGES::Object::API::BaseAttribute'
    });
    my $TestObject = LoadRootObject->insertChild( $Class1, { Alias => 'Test' } );
    eval { $TestObject->set({ 'NoColumn' => 1.2 }); };
    ok( ExistsError(), 'set attribute without a column' );
    is( GetError->code, 'COLUMN_NOT_FOUND', 'error code COLUMN_NOT_FOUND' );

    # EPG-14476: saves empty string in DB
    # FilePathPart - BaseAttribute
    # A4String - DefaultAttribute
    $Class3->insertAttribute({ Alias => 'A4String', Type => 'String', Package => $ATTRIBUTE_PACKAGE });
    my $TestObject2 = $TestObject->insertChild($Class3, { Alias => 'Test2', FilePathPart => ' ', A4String => ' ' });
    is( $TestObject2->get('FilePathPart'), undef, 'BaseAttribute after insertChild with \'\' is null');
    is( $TestObject2->get('A4String'), undef, 'DefaultAttribute after insertChild with \'\' is null');

    $TestObject2->set({'FilePathPart' => ' ', 'A4String' => ' '});
    ok( !defined $TestObject2->get('FilePathPart'), 'BaseAttribute after set with \'\' is null');
    ok( !defined $TestObject2->get('A4String'), 'DefaultAttribute after set with \'\' is null');

    my $TestObject3 = $TestObject->insertChild($Class3, { Alias => 'Test3', FilePathPart => undef, A4String => undef });
    ok( !defined $TestObject3->get('FilePathPart'), 'BaseAttribute after insertChild with undef is null');
    ok( !defined $TestObject3->get('A4String'), 'DefaultAttribute after insertChild with undef is null');

    $TestObject3->set({'FilePathPart' => undef, 'A4String' => undef});
    ok( !defined $TestObject3->get('FilePathPart'), 'BaseAttribute after set with undef is null');
    ok( !defined $TestObject3->get('A4String'), 'DefaultAttribute after set with undef is null');

    # EPG-18072: default values for scalar attributes
    my $A5Bool = $Class3->insertAttribute({ Alias => 'A5Bool', Type => 'Boolean',
        Package => $ATTRIBUTE_PACKAGE
    });
    my $A6Bool = $Class3->insertAttribute({ Alias => 'A6Bool', Type => 'Boolean',
        DefaultValue => 1,
        Package => $ATTRIBUTE_PACKAGE
    });
    my $A7Bool = $Class3->insertAttribute({ Alias => 'A7Bool', Type => 'Boolean',
        DefaultValue => 0,
        Package => $ATTRIBUTE_PACKAGE
    });

    is( $TestObject3->get('A5Bool'), 0, 'Boolean attribute without default value' );
    is( $TestObject3->get('A6Bool'), 1, 'Boolean attribute with default value 1' );
    is( $TestObject3->get('A7Bool'), 0, 'Boolean attribute with default value 0' );

    my @BoolAttibutes = qw(A5Bool A6Bool A7Bool);
    $TestObject3->set({ $_ => 0 }) foreach @BoolAttibutes;
    is( $TestObject3->get($_), 0, "Attribute $_ set to 0" ) foreach @BoolAttibutes;
    ok( !ExistsBooleanAttribute( $TestObject3->id, $A5Bool->id ), 'A5Bool value=0 does not exist' );
    ok( ExistsBooleanAttribute( $TestObject3->id, $A6Bool->id ), 'A6Bool value=0 exists' );
    ok( !ExistsBooleanAttribute( $TestObject3->id, $A7Bool->id ), 'A7Bool value=0 does not exist' );

    $TestObject3->set({ $_ => 1 }) foreach @BoolAttibutes;
    is( $TestObject3->get($_), 1, "Attribute $_ set to 1" ) foreach @BoolAttibutes;
    ok( ExistsBooleanAttribute( $TestObject3->id, $A5Bool->id ), 'A5Bool value=1 exists' );
    ok( !ExistsBooleanAttribute( $TestObject3->id, $A6Bool->id ), 'A6Bool value=1 does not exist' );
    ok( ExistsBooleanAttribute( $TestObject3->id, $A7Bool->id ), 'A7Bool value=1 exists' );

    $TestObject3->set({ $_ => undef }) foreach @BoolAttibutes;
    is( $TestObject3->get($_), 0, "Attribute $_ set to undef" ) foreach @BoolAttibutes;
    ok( !ExistsBooleanAttribute( $TestObject3->id, $A5Bool->id ), 'A5Bool value=0 does not exist' );
    ok( ExistsBooleanAttribute( $TestObject3->id, $A6Bool->id ), 'A6Bool value=0 exists' );
    ok( !ExistsBooleanAttribute( $TestObject3->id, $A7Bool->id ), 'A7Bool value=0 does not exist' );

    $TestObject3->set({ $_ => 'ABC' }) foreach @BoolAttibutes;
    is( $TestObject3->get($_), 1, "Attribute $_ set to ABC" ) foreach @BoolAttibutes;
    ok( ExistsBooleanAttribute( $TestObject3->id, $A5Bool->id ), 'A5Bool value=1 exists' );
    ok( !ExistsBooleanAttribute( $TestObject3->id, $A6Bool->id ), 'A6Bool value=1 does not exist' );
    ok( ExistsBooleanAttribute( $TestObject3->id, $A7Bool->id ), 'A7Bool value=1 exists' );

    is( $TestObject3->get('A1Float'), undef, 'Float attribute without default value' );
    is( $TestObject3->get('A2Float'), FLOAT_VALUE, 'Float attribute with default value' );

    my @FloatAttributes = qw(A1Float A2Float A3Float);
    $TestObject3->set({ $_ => FLOAT_VALUE2 }) foreach @FloatAttributes;
    is( $TestObject3->get($_), FLOAT_VALUE2, "Attribute $_ set to FLOAT_VALUE2" ) foreach @FloatAttributes;
    ok( ExistsFloatAttribute( $TestObject3->id, $TestObject3->class->attribute($_)->id ), "$_ value=FLOAT_VALUE2 exists" ) foreach @FloatAttributes;

    $TestObject3->set({ $_ => FLOAT_VALUE }) foreach @FloatAttributes;
    is( $TestObject3->get($_), FLOAT_VALUE, "Attribute $_ set to FLOAT_VALUE" ) foreach @FloatAttributes;
    ok( ExistsFloatAttribute( $TestObject3->id, $TestObject3->class->attribute($_)->id ), "$_ value=FLOAT_VALUE exists" ) foreach qw( A1Float A3Float);
    ok( !ExistsFloatAttribute( $TestObject3->id, $A6Bool->id ), 'A6Bool value=FLOAT_VALUE does not exist' );

    $TestObject3->set({ $_ => undef }) foreach @FloatAttributes;
    is( $TestObject3->get($_), undef, "Attribute $_ set to undef" ) foreach qw( A1Float A3Float);
    is( $TestObject3->get('A2Float'), FLOAT_VALUE, 'Attribute A2Float set to undef' );
    ok( !ExistsFloatAttribute( $TestObject3->id, $TestObject3->class->attribute($_)->id ), "$_ value=undef does not exist" ) foreach @FloatAttributes;

    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C1 C2 C3 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }

    my $System = LoadRootObject();
    $System->child('Test')->delete if $System->existsChild('Test');
    return 1;
}

sub Init {
    Cleanup();

    my $ClassObject = LoadClassByAlias( 'Object' );

    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'A1Float', Type => 'Float', Package => $ATTRIBUTE_PACKAGE, 'ExportLevel'=>'2.2' });

    my $Class2 = $Class1->insertSubClass({ Alias => 'C2' });
    $Class2->insertAttribute({ Alias => 'A2Float', Type => 'Float', Package => $ATTRIBUTE_PACKAGE,
        ExportLevel => undef,
        DefaultValue => FLOAT_VALUE,
    });

    my $Class3 = $Class2->insertSubClass({ Alias => 'C3' });
    $Class3->insertAttribute({ Alias => 'A3Float', Type => 'Float', Package => $ATTRIBUTE_PACKAGE });

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
