use Test::More tests => 47;
use strict;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountriesByCode2 );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Shop::API::Address qw ( CheckVATID UpdateVATID IsTaxModelNet GetTaxCountryIDForShop );
use DE_EPAGES::Shop::API::Table::ShopCountry qw ( GetInfoShopCountriesByShopID );

my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    Init();

    my $DE = GetInfoCountriesByCode2('DE')->[0]->{'CountryID'};
    ok (CheckVATID('DE 123 456 789', $DE) == 1, '\'DE 123 456 789\' is valid in DE');
    ok (CheckVATID('AT 123 456 789', $DE) == -1, '\'AT 123 456 789\' is not in DE');
    ok (CheckVATID('DE 123 456 789 012', $DE) == 0, '\'DE 123 456 789 012\' is not valid in DE');
    ok (CheckVATID('DE 123-456-789', $DE) == 1, '\'DE 123-456-789\' is valid in DE');

    my $AT = GetInfoCountriesByCode2('AT')->[0]->{'CountryID'};
    ok (CheckVATID('AT U 123 456 78', $AT) == 1, '\'AT U 123 456 78\' is valid in AT');
    ok (CheckVATID('DE 123 456 789', $AT) == -1, '\'DE 123 456 789\' is not in AT');
    ok (CheckVATID('AT 123 456 789', $AT) == 0, '\'AT 123 456 789\' is not valid in AT');

    my $CH = GetInfoCountriesByCode2('CH')->[0]->{'CountryID'};
    ok (CheckVATID('CH 123 456 789', $CH) == 1, '\'CH 123 456 789\' is valid in CH');
    ok (CheckVATID('XY 123 456 789', $CH) == 1, '\'XY 123 456 789\' is valid in CH');
    ok (CheckVATID('AT 123 456 789', $CH) == 1, '\'AT 123 456 789\' is valid in CH');
    ok (CheckVATID('DE 123 456 789', $CH) == 1, '\'DE 123 456 789\' is valid in CH');

    my $AU = GetInfoCountriesByCode2('AU')->[0]->{'CountryID'};
    ok (CheckVATID('XYZ', $AU) == 1, 'any VAT-ID is valid for non-EU countries');
    ok (CheckVATID('XYZ') == 1, 'any VAT-ID is valid if no country is specified');

    my $GR = GetInfoCountriesByCode2('GR')->[0]->{'CountryID'};
    ok (CheckVATID('123 456 789', $GR) == -1, '\'123 456 789\' is not valid in GR without countrycode');
    ok (CheckVATID('GR 123 456 789', $GR) == -1, '\'GR 123 456 789\' is not in GR');
    ok (CheckVATID('EL 123 456 789', $GR) == 1, '\'EL 123 456 789\' is valid in GR');

    eval { CheckVATID(undef, $DE) };
    ok( ExistsError(), 'CheckVATID - missing VATID');
    is( GetError()->code, 'PARAMETER_MISSING', 'CheckVATID - missing VATID - error code');

    is (UpdateVATID('DE 123 456 789', $DE), 'DE123456789', 'VATID with countrycode');
    is (UpdateVATID('EU 123 456 789', $DE), 'EU123456789', 'VATID with EU countrycode');
    is (UpdateVATID('123 456 789', $DE), 'DE123456789', 'VATID without countrycode for DE');
    is (UpdateVATID('123 456 789', $AT), 'AT123456789', 'VATID without countrycode for AT');
    is (UpdateVATID('123 456 789'), '123456789', 'VATID without country id');
    is (UpdateVATID('EL 123 456 789', $GR), 'EL123456789', 'VATID with countrycode for GR');
    is (UpdateVATID('123 456 789', $GR), 'EL123456789', 'VATID without countrycode for GR');
    is (UpdateVATID('GR 123 456 789', $GR), 'ELGR123456789', 'VATID without countrycode for GR');
    is (UpdateVATID('DE 123 456 789', $GR), 'DE123456789', 'VATID with wrong countrycode for GR');

    is (UpdateVATID('DE 123-456-789', $DE), 'DE123456789', 'VATID with character "-"');

    # test error conditions
    eval { UpdateVATID(undef, $DE) };
    ok( ExistsError(), 'UpdateVATID - missing VATID');
    is( GetError()->code, 'PARAMETER_MISSING', 'UpdateVATID - missing VATID - error code');

    # test VATIDs (for switching customer to gross or net)
    my $Address = LoadObjectByPath('/Test/Address');
    my $Shop = LoadObjectByPath('/Test/TestShop1');
    my $ES = GetInfoCountriesByCode2('ES')->[0]->{'CountryID'};
    my $AR = GetInfoCountriesByCode2('AR')->[0]->{'CountryID'};
    my $GB = GetInfoCountriesByCode2('GB')->[0]->{'CountryID'};

    my $ShopCountryID = $Shop->get('Address')->get('CountryID');
    ok( !defined $ShopCountryID, 'new shop has not selected a country yet' );
    is( $Shop->defaultCountry, 826, 'default shop country: GB' );
    is( GetTaxCountryIDForShop( $Shop ), 826, 'use shop default country for tax (GB)' );

    $Address->set({'CountryID'=> $ES , 'VATID'    => 'ES 25719487K', });
    ok( !IsTaxModelNet($Shop, $Address), '"ES 25719487K" is a private NIF --> gross');

    $Address->set({'CountryID'=> $ES , 'VATID'    => 'ES L25719487K', });
    ok( !IsTaxModelNet($Shop, $Address), '"ES L25719487K" is a private NIF --> gross');

    $Address->set({'CountryID'=> $ES , 'VATID'    => 'ES B64817133', });
    ok( IsTaxModelNet($Shop, $Address), '"ES B64817133" is a business NIF --> net');

    $Address->set({'CountryID'=> $DE , 'VATID'    => 'DE 123 456 789', });
    ok( IsTaxModelNet($Shop, $Address), '"DE 123 456 789" is a business VAT ID --> net');

    $Address->set({'CountryID'=> $AR , 'VATID'    => '', });
    ok( IsTaxModelNet($Shop, $Address), 'non-EU foreigner (private) --> net');

    $Address->set({'CountryID'=> $AR , 'VATID'    => 'AG12345', });
    ok( IsTaxModelNet($Shop, $Address), 'non-EU foreigner (business) --> net');

    $Address->set({'CountryID'=> $ES , 'VATID'    => '', });
    ok( !IsTaxModelNet($Shop, $Address), 'EU foreigner without VAT ID --> gross');

    $Address->set({'CountryID'=> $GB , 'VATID'    => '', });
    ok( !IsTaxModelNet($Shop, $Address), 'Inland customer without VAT ID --> gross');

    # non-EU shop: set default country to Argentina
    my $ahShopCountries = GetInfoShopCountriesByShopID( $Shop->id );
    foreach my $hShopCountry (@$ahShopCountries) {
        if( $hShopCountry->{IsDefault} ) {
            $Shop->updateCountry( $hShopCountry->{CountryID}, $hShopCountry->{Position}, 0 );
        }
    }

    $Shop->addCountry( $AR, 100, 1 );

    is( $Shop->defaultCountry, $AR, 'default shop country: AR' );
    is( GetTaxCountryIDForShop( $Shop ), $AR, 'use shop default country for tax (AR)' );

    $Address->set({'CountryID'=> $AR , 'VATID'    => '', });
    ok( !IsTaxModelNet($Shop, $Address), 'non-eu-shop: Inland customer without VAT ID --> gross');

    $Address->set({'CountryID'=> $AR , 'VATID'    => 'AG12345', });
    ok( !IsTaxModelNet($Shop, $Address), 'non-eu-shop: Inland customer with VAT ID --> gross'); # because we can't verify the VAT ID

    $Address->set({'CountryID'=> $ES , 'VATID'    => '', });
    ok( IsTaxModelNet($Shop, $Address), 'non-eu-shop: Foreigner without VAT ID --> net');

    $Address->set({'CountryID'=> $ES , 'VATID'    => 'ES B64817133', });
    ok( IsTaxModelNet($Shop, $Address), 'non-eu-shop: Foreigner with VAT ID --> net');

    Cleanup();

    DisconnectSite();
}

sub Cleanup {

    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    return 1;
}

sub Init {
    Cleanup();

    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );

    $Folder->insertChild( 'Address', {
        'Alias'   => 'Address',
        'Street'  => 'Leutragraben',
        'City'    => 'Jena',
        'Zipcode' => '07740',
        'Company' => 'epages',
    });
    return 1;
}


RunScript(Sub=>\&Main, Debug=>1);


#'CY' => '[0-9]{8}[A-Za-z]',
#'IE' => '[0-9][A-Za-z0-9\\+\\*][0-9]{5}[A-Za-z]',
#'GB' => '([0-9]{3} ?[0-9]{4} ?[0-9]{2}|[0-9]{3} ?[0-9]{4} ?[0-9]{2} ?[0-9]{3}|GD[0-9]{3}|HA[0-9]{3})',
#'NL' => '[0-9]{9}B[0-9]{2}',
#'AT' => 'U[0-9]{8}',
#'BE' => '0[0-9]{9}',
#'EL' => '[0-9]{9}',
#'EE' => '[0-9]{9}',
#'LV' => '[0-9]{11}',
#'DK' => '[0-9]{2} ?[0-9]{2} ?[0-9]{2} ?[0-9]{2}',
#'PT' => '[0-9]{9}',
#'FI' => '[0-9]{8}',
#'DE' => '[0-9]{9}',
#'SE' => '[0-9]{12}',
#'BG' => '[0-9]{9,10}',
#'LU' => '[0-9]{8}',
#'SI' => '[0-9]{8}',
#'MT' => '[0-9]{8}',
#'LT' => '([0-9]{9}|[0-9]{12})',
#'RO' => '[0-9]{2,10}',
#'IT' => '[0-9]{11}',
#'PL' => '[0-9]{10}',
#'CZ' => '[0-9]{8,10}',
#'ES' => '([A-Za-z0-9][0-9]{7}[A-Za-z0-9])',
#'SK' => '[0-9]{10}',
#'HU' => '[0-9]{8}',
#'FR' => '[A-Za-z0-9]{2} ?[0-9]{9}'