use Test::More tests => 25;
use strict;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectToSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime AddDuration SubtractDuration);
use DE_EPAGES::Core::API::Net qw ( GetHttpResponse );

local $ENV{'EPAGES_TESTSITE'} = 'Site';

sub Main {
    ConnectToSite();

    # Clean up the shop
    ok( !Cleanup(), 'cleaned' );

    my $System   = LoadRootObject();
    my $Store    = $System->folder('Stores')->child('Store');
    my $Provider = $System->folder('Providers')->child('Distributor');
    my $ShopType = $Provider->folder('ShopTypes')->child('Demo');

    my $ShopRef = $Provider->insertShop({
        'Alias'     => 'Test',
        'ShopAlias' => 'Test',
        'Store'     => $Store,
        'ShopType'  => $ShopType,
    });

    # Test if shop was created
    ok (defined $ShopRef, 'shop created');

    ok ( $ShopRef->get('ExistsRemoteShop'),'shop exists' );

    my $RemoteShop = $ShopRef->remoteShop;

    # Test the update of a single Super Merchant value
    $RemoteShop->updateSuperMerchant( { 'Name' => 'TestName' } );

    my $hMerchantValues = $RemoteShop->getInfoSuperMerchant();
    is( $hMerchantValues->{'Name'},
        'TestName', 'Test update "Name" of SuperMerchant' );

    ## no critic (ValuesAndExpressions::RequireInterpolationOfMetachars)

    # Test the update of more than one values of the super merchant
    $RemoteShop->updateSuperMerchant(
        {
            'Alias'                => 'TestAlias',
            'EMail'                => 'test@epages.com',
            'Name'                 => 'TestName2',
            'LocaleID'             => 'fr_FR',
            'DeleteConfirmation'   => '1',
            'IsLoginCookieAllowed' => '1',
            'ShowFunctionHelp'     => '1',
        }
    );

    ## use critic


    $hMerchantValues = $RemoteShop->getInfoSuperMerchant();

    is( $hMerchantValues->{'Name'},
        'TestName2', 'Test2 update "Name" of SuperMerchant' );
    is( $hMerchantValues->{'Alias'},
        'TestAlias', 'Test update "Alias" of SuperMerchant' );
    is( $hMerchantValues->{'EMail'},
        'test@epages.com', 'Test update "EMail" of SuperMerchant' );
    is( $hMerchantValues->{'LocaleID'},
        'fr_FR', 'Test update "LocaleID" of SuperMerchant' );
    is( $hMerchantValues->{'DeleteConfirmation'},
        '1', 'Test update "DeleteConfirmation" of SuperMerchant' );
    is( $hMerchantValues->{'IsLoginCookieAllowed'},
        '1', 'Test update "IsLoginCookieAllowed" of SuperMerchant' );
    is( $hMerchantValues->{'ShowFunctionHelp'},
        '1', 'Test update "ShowFunctionHelp" of SuperMerchant' );

    # Matching string to identify if the user is logged in or not
    my $notLoggedInString = 'id="LoginButton"';
    my $LoggedInString = 'class="LogoutLink"';

    my $hMboInfo = $RemoteShop->createSuperMerchantSession();

# Adding the Login and Password to MBO Link to simluate filling out the login form
# Check, that this will lead to no access
    my $Response =
      GetHttpResponse( $hMboInfo->{WebUrlAdminSSL}
          . '&ChangeAction=SaveLoginForm&Login=TestAlias2&Password=TestPassword03'
      );
    ok(
        $Response->content =~ /\Q$notLoggedInString\E/
          && !( $Response->content =~ /\Q$LoggedInString\E/ ),
        'Login before updated Super merchant - access denied'
    );

# Another upate test of more than one values of the super merchant to be sure it
# was really set and not just the return of the default setting
    $RemoteShop->updateSuperMerchant(
        {
            'Alias'                => 'TestAlias2',
            'EMail'                => 'test2@epages.com',
            'Password'             => 'TestPassword03',
            'Name'                 => 'TestName3',
            'LocaleID'             => 'en_GB',
            'DeleteConfirmation'   => '0',
            'IsLoginCookieAllowed' => '0',
            'ShowFunctionHelp'     => '0',
        }
    );

# Check if now after update the super merchants alias and passwort the access will be allowed

    $Response =
      GetHttpResponse( $hMboInfo->{WebUrlAdminSSL}
          . '&ChangeAction=SaveLoginForm&Login=TestAlias2&Password=TestPassword03'
      );
    ok(
        $Response->content =~ /\Q$notLoggedInString\E/
          && !( $Response->content =~ /\Q$LoggedInString\E/ ),
        'Login after updated Super merchant - access allowed'
    );

    # Check a second time that all values were set correct
    $hMerchantValues = $RemoteShop->getInfoSuperMerchant();
    is( $hMerchantValues->{'Name'},
        'TestName3', 'Test3 update "Name" of SuperMerchant' );
    is( $hMerchantValues->{'Alias'},
        'TestAlias2', 'Test2 update "Alias" of SuperMerchant' );
    is( $hMerchantValues->{'EMail'},
        'test2@epages.com', 'Test2 update "EMail" of SuperMerchant' );
    is( $hMerchantValues->{'LocaleID'},
        'en_GB', 'Test2 update "LocaleID" of SuperMerchant' );
    is( $hMerchantValues->{'DeleteConfirmation'},
        '0', 'Test2 update "DeleteConfirmation" of SuperMerchant' );
    is( $hMerchantValues->{'IsLoginCookieAllowed'},
        '0', 'Test2 update "IsLoginCookieAllowed" of SuperMerchant' );
    is( $hMerchantValues->{'ShowFunctionHelp'},
        '0', 'Test2 update "ShowFunctionHelp" of SuperMerchant' );


    $hMboInfo  = $RemoteShop->createSuperMerchantSession();
    $Response = GetHttpResponse($hMboInfo->{WebUrlAdminSSLWithSessionId});

    # Test that creating a session for super merchant without an valid GrantServiceAccessUntil-date will lead to access denied
    ok(
        $Response->content =~ /\Q$notLoggedInString\E/
          && !( $Response->content =~ /\Q$LoggedInString\E/ ),
        'GrantServiceAccessUntil undefinded - access denied'
    );

    # set a valid date in the past and check that access will be denied
    my $Parser = DE_EPAGES::XML::API::DateTimeFormatXML->new;
    my $hValues->{'GrantServiceAccessUntil'} = $Parser->format_datetime(
        SubtractDuration( GetCurrentDateTime(), 'days', 1 ) );
    $RemoteShop->set($hValues);
    $hMboInfo  = $RemoteShop->createSuperMerchantSession();
    $Response = GetHttpResponse($hMboInfo->{WebUrlAdminSSLWithSessionId});

    ok(
        $Response->content =~ /\Q$notLoggedInString\E/
          && !( $Response->content =~ /\Q$LoggedInString\E/ ),
        'GrantServiceAccessUntil Date in the Past - access denied'
    );

    # set a valid date in the future
    $hValues->{'GrantServiceAccessUntil'} = $Parser->format_datetime(
        AddDuration( GetCurrentDateTime(), 'days', 1 ) );
    $RemoteShop->set($hValues);
    $hMboInfo  = $RemoteShop->createSuperMerchantSession();

    # try to access with a wrong session id
    $Response = GetHttpResponse($hMboInfo->{WebUrlAdminSSLWithSessionId}.'XXX');
    ok(
        $Response->content =~ /\Q$notLoggedInString\E/
          && !( $Response->content =~ /\Q$LoggedInString\E/ ),
        'GrantServiceAccessUntil with wrong session - access denied'
    );


    # try to acces with the right session id
    $Response = GetHttpResponse($hMboInfo->{WebUrlAdminSSLWithSessionId});

        ok(
        !($Response->content =~ /\Q$notLoggedInString\E/)
          && $Response->content =~ /\Q$LoggedInString\E/ ,
        'GrantServiceAccessUntil Date in the Future, access allowed'
    );

    # clean up
    ok (Cleanup(), 'cleaned');
    DisconnectSite();
}

RunScript(
    Sub => \&Main
);

sub Cleanup {
    my $System   = LoadRootObject();
    my $Provider = $System->folder('Providers')->child('Distributor');
    if ($Provider->folder('ShopRefs')->existsChild('Test')) {
        $Provider->folder('ShopRefs')->child('Test')->deleteRemoteShop;
        $Provider->folder('ShopRefs')->child('Test')->delete;
        return 1;
    } else {
        return 0;
    }
}
