## no critic(ProhibitExcessMainComplexity)
use 5.010;
use strict;
use warnings;
## no critic( RequireCorrectImports RequireTestLabels )
## no critic( RequireExistsError RequireExtendedFormatting) EP
use LWP::UserAgent;
use XML::LibXML;
use Getopt::Std;

use Test::More;
use Test::Exception;

use DE_EPAGES::Twenga::API::Impl;

use constant CTXML => 'text/xml; charset=UTF-8';
use constant LINE  => '-' x 70 . "\n";

sub ERRBS {
    my $rsp = shift;
    return LINE, $rsp->request->as_string, LINE, $rsp->as_string, LINE,
    'Error talking to Twenga over the net; might be temporary; giving up.';
}

# ==== What is this supposed to do? ====================================
# Simple script to test Twenga API calls.
# Just test them one after the other.
# There is a little bit of configurability thanks to getopts.
# And there's some quirkiness with some of the Twenga API details.
# Just take it as it is, it has to be that way.



# ==== utility methods =================================================

sub xml {
    my( $twenga, $rsp, $msg_rex ) = @_;
    $msg_rex ||= qr/^true$/;
    # old style
    my $doc = eval { XML::LibXML->load_xml( string => $rsp->content ) };
    ok $doc, 'parsed content into document';
    my $xpc = XML::LibXML::XPathContext->new( $doc );
    my @nodes = $xpc->findnodes( '/response' );
    is scalar @nodes, 1;
    @nodes = $xpc->findnodes( '/response/message' );
    is scalar @nodes, 1;
    my $val = $xpc->findvalue( '/response/message' );
    like $val, $msg_rex, $msg_rex;
    # new style
    my $hash = $twenga->data( $rsp );
    0 and diag explain $hash;
    like $hash->{content}, $msg_rex, $msg_rex;
    ok ! $hash->{error};
    ok ! $hash->{errormessage};
    return;
}

# ==== main ============================================================

my $VRB;
my $mrcacc;
{
    getopts 'hvK:', \my %opt;
    if ( $opt{h} ) {
        print <<'EOS';
  -h            Hilfe
  -v            verbos
  -K            Konto
EOS
        exit 0;
    }
    $VRB = 1 if $opt{v};
    # which account?
    my %merchant_accounts = (
        tmetzinger  =>
    [qw[tmetzinger@epages.de   8uakpusr    aa541ef3422583424d4ad9bb4681b2f0 FR]],
        balu        =>
    [qw[balu@epages.de         n/oxklbb    0ffaca95e3e5242ba1097ad8a9a6e95d DE]],
        test        =>
    [qw[test@epages.de         @4hbnatv    2d2c8394e31101a261abf1784302bf75 GB]],
    );
    $opt{K} //= 'balu'; # default account
    $mrcacc = $merchant_accounts{ $opt{K} } or die "kein Konto [$opt{K}]!";
    diag "Konto: @$mrcacc";
}

# Partner accounts
# ============================
# prod_epages   D5vuV9Z98n
# dev_epages    sxwoxhfp
# ============================

# my $api_user = 'prod_epages';
# my $api_pswd = 'D5vuV9Z98n';

my $twenga = Twenga->new(
    partner_auth_key => '1f2027e88b6964f3b952212b6d15397ae2a7d74a',
#   test_mode        => 1,
#   partner_username => $api_user,
#   partner_password => $api_pswd
);
isa_ok $twenga, 'Twenga';

# md5 as hex, then as Base64 (doesn't make much sense, but it's like that)
is $twenga->partner_auth_key_b64,
    'MWYyMDI3ZTg4YjY5NjRmM2I5NTIyMTJiNmQxNTM5N2FlMmE3ZDc0YQ=='
    or die 'PARTNER_AUTH_KEY falsch';

my $api_url = $twenga->api_url;
is $api_url, 'https://rts.twenga.com/api/', 'API URL ok';

my $twenga_url = qr#^$api_url\w+/\w+/?\?.*PARTNER_AUTH_KEY#;

my $ua = LWP::UserAgent->new( keep_alive => 1 );
my $rsp;

{   # test Site/GetSubscriptionLink call w/o auth (will work)
    my $getsublink = $twenga->make_req( Site => GetSubscriptionLink => {
            country     => 'FR',
            feed_url    => 'http://bla.de/feed.txt',
            site_url    => 'http://bla.de/',
        } );
    isa_ok $getsublink, 'HTTP::Request';
    like $getsublink->uri, $twenga_url, 'URL ~ ok';
    $VRB and diag $getsublink->as_string;
    ok ! $getsublink->header( 'Authorization' ), 'no Authorization header';

    $rsp = $ua->request( $getsublink );
    ok $rsp->is_success, 'response success' or die ERRBS($rsp);
    is $rsp->header('content-type'), CTXML or die ERRBS($rsp);
    $VRB and diag $rsp->as_string;
}



{   # test Site/Exist call w/o auth (won't work)
    throws_ok { $twenga->make_req( Site => Exist => {} ) }
    qr/\bAttribute\b.*\bkey\b/;

    xml( $twenga, $rsp, qr#^https://rts.twenga.(?:com|fr)/subscribe/\?subscription_key=\w+$# );
}

# add credentials for auth and retry
$twenga->merchant_login(    $mrcacc->[0] );
$twenga->merchant_password( $mrcacc->[1] );
$twenga->merchant_hashkey(  $mrcacc->[2] );

{   # test Site/Exist call with auth
    my $site_exist = $twenga->make_req( Site => Exist => { } );
    isa_ok $site_exist, 'HTTP::Request';
    like $site_exist->uri, $twenga_url, 'URL ~ ok';
    my $auth = $site_exist->header( 'Authorization' );
    ok $auth, 'has Authorization header';
    is $auth, "Basic NTVkNjE1YTk1N2RjYzcwYTRiYzFhMTAwYmZjMDcwOTI6MTEyZjVhMWNlZjZmOThmNTMwYjY0ZjU0MDBhYmEwMzI=\n";
    $VRB and diag $site_exist->as_string;

    $rsp = $ua->request( $site_exist );
    ok $rsp->is_success, 'response success' or die ERRBS($rsp);
    is $rsp->header('content-type'), CTXML or die ERRBS($rsp);
    $VRB and diag $rsp->as_string;
}


{   # test Site/Activate call
    my $site_act = $twenga->make_req( Site => Activate => { } );
    ok $site_act->header( 'Authorization' );
    $rsp = $ua->request( $site_act );
    ok $rsp->is_success, 'response success' or die ERRBS($rsp);
    is $rsp->header('content-type'), CTXML or die ERRBS($rsp);
    $VRB and diag $rsp->as_string;

    xml( $twenga, $rsp );
}


{   # test Site/AddFeed call
    my $site_addfeed = $twenga->make_req( Site => AddFeed => {
            feed_url => 'http://blubbelblub.de/gurkensalat',
        } );
    ok $site_addfeed->header( 'Authorization' );
    $rsp = $ua->request( $site_addfeed );
    ok $rsp->is_success, 'response success' or die ERRBS($rsp);
    is $rsp->header('content-type'), CTXML or die ERRBS($rsp);
    $VRB and diag $rsp->as_string;

    xml( $twenga, $rsp );
}



{   # test Site/GetTrackingScript call
    my $site_getscript = $twenga->make_req( Site => GetTrackingScript => {
            basket_id       => 99,
            order_id        => 3456789,
            cli_email       => 'mludwig@epages.de',
            total_ht        => 88.79,
            user_id         => 878787,
        });
    ok $site_getscript->header( 'Authorization' );
    $rsp = $ua->request( $site_getscript );
    ok $rsp->is_success, 'response success' or die ERRBS($rsp);
    is $rsp->header('content-type'), CTXML or die ERRBS($rsp);
    $VRB and diag $rsp->as_string;

    xml( $twenga, $rsp, qr#^\s*<script language="javascript" type="text/javascript"# );
    my $content = $rsp->decoded_content;
    like   $content, qr#\bhttps://#, 'https found';
    unlike $content, qr#\bhttp://#, 'no http present';
}

done_testing;
