use strict;
use Test::More tests => 56;
use DE_EPAGES::Address::API::Address qw( CheckZipcode );
use DE_EPAGES::Address::DAL::Address;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::DataCache::API::Cache qw( UpdateLocalConnection );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath LoadClassByAlias ExistsObjectByPath );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountriesByCode2 );

local $ENV{'EPAGES_TESTSITE'} = "Store" unless $ENV{'EPAGES_TESTSITE'};

sub Main {
    ConnectTestSite();
    ok( Init(), "initialization" );

    my $Object = LoadObjectByPath('O1');
    ok( defined $Object, 'address exists');
    ok( $Object->get('Company') eq 'epages', 'company epages');
    ok( $Object->get('City') eq 'Jena', 'in jena');
    ok( $Object->get('Street') eq 'Leutragraben 2-4', 'street address');
    ok( $Object->get('HouseNumber') eq '2-4', 'street address');
    ok( $Object->get('StreetOnly') eq 'Leutragraben', 'street address');

    $Object->set({'Company'=>undef});
    ok( (not defined DAL_GetInfoAddressPerson($Object->id)) , 'person address deleted');

    $Object->set({'Company'=>'epages'});
    ok( $Object->get('Company') eq 'epages', 'company epages');

    $Object->set({'L1' => 'deutscher text'}, GetPKeyLanguageByCode('de'));
    $Object->set({'L1' => 'english text'}, GetPKeyLanguageByCode('en'));

    my $ObjectClone = $Object->clone({'Alias'=>'O2'});
    ok( defined $ObjectClone, 'clone exists');
    ok( $ObjectClone->get('Company') eq 'epages', 'clone company epages');
    ok( $ObjectClone->get('L1', GetPKeyLanguageByCode('en')) eq 'english text', 'clone localize');

    #check zipcode
    my $Country = GetInfoCountriesByCode2('DE')->[0]->{'CountryID'};
    ok (CheckZipcode('01234', $Country), "valid german zipcode 01234");
    ok (CheckZipcode('10234', $Country), "valid german zipcode 10234");
    ok (!CheckZipcode('012345', $Country), "invalid german zipcode 012345");
    ok (!CheckZipcode('00234', $Country), "invalid german zipcode 00234");
    ok (!CheckZipcode('102345', $Country), "invalid german zipcode 102345");
    ok (!CheckZipcode('A0234', $Country), "invalid german zipcode A0234");

    $Country = GetInfoCountriesByCode2('AT')->[0]->{'CountryID'};
    ok (CheckZipcode('1234', $Country), "valid austrian zipcode 1234");
    ok (!CheckZipcode('123', $Country), "invalid austrian zipcode 123");
    ok (!CheckZipcode('0123', $Country), "invalid austrian zipcode 0123");

    $Country = GetInfoCountriesByCode2('US')->[0]->{'CountryID'};
    ok (CheckZipcode('12345', $Country), "valid us zipcode 12345");
    ok (CheckZipcode('01234-5678', $Country), "valid us zipcode 01234-5678");
    ok (CheckZipcode('12345 4321', $Country), "valid us zipcode 12345 4321");
    ok (!CheckZipcode('123', $Country), "invalid us zipcode 123");
    ok (!CheckZipcode('12345 43214', $Country), "invalid us zipcode 12345 43214");

    ok (CheckZipcode('12345 43214'), "missing country id");
    ok (CheckZipcode('12345 43214', "\t"), "empty country id");
    ok (CheckZipcode(undef, 840), "empty zipcode");
    ok (CheckZipcode(' ', 840), "empty zipcode");
    ok (CheckZipcode('12345 43214', 732), "unknown country id");

    $Country = GetInfoCountriesByCode2('AU')->[0]->{'CountryID'};
    ok (!CheckZipcode('08343332222234324', $Country), "invalid australien zipcode 08343332222234324");
    ok (!CheckZipcode('00234333', $Country), "invalid australien zipcode 00234333");
    ok (!CheckZipcode('B0834333222sadf2234324', $Country), "invalid australien zipcode B0834333222sadf2234324");
    ok (CheckZipcode('1234', $Country), "valid australien zipcode 1234");

    $Country = GetInfoCountriesByCode2('BR')->[0]->{'CountryID'};
    ok (!CheckZipcode('0635543035235', $Country), "invalid brazil zipcode 0635543035235");
    ok (!CheckZipcode('44229-256asd5657', $Country), "invalid brazil zipcode 44229-256asd5657");

    $Country = GetInfoCountriesByCode2('NL')->[0]->{'CountryID'};
    ok (CheckZipcode('3911 KT', $Country), "valid netherland zipcode 3911 KT");
    ok (CheckZipcode('3911', $Country), "valid netherland zipcode 3911");
    ok (CheckZipcode('3911KT', $Country), "valid netherland zipcode 3911KT");
    ok (!CheckZipcode('3911 KTA', $Country), "invalid netherland zipcode 3911 KTA");
    ok (!CheckZipcode('0911 KT', $Country), "invalid netherland zipcode 0911 KT");

    eval { CheckZipcode('12345 43214', -1) };
    ok( ExistsError() && GetError()->code eq 'NoResults_Country', "invalid country id");

    # Test data encryption

    # Setup database; usually each db section should contain an "key" entry where the aes can be found, like:
    #
    #   [Connection]
    #   datasource=dbi:mysql
    #   login=usr_sitedb
    #   passwd=epages
    #   active=1
    #   Database=sitedb
    #   StoreType=Site
    #   key=hst58a42hvbt8671loegysq87n4mc7z1
    #
    # if not, we add one key here.
    my $dbi = GetCurrentDBHandle();
    $dbi->{'key'} = "hst58a42hvbt8671loegysq87n4mc7z1" unless $dbi->{'key'};

    my $System = LoadRootObject();
    foreach my $TryNumber ( 1 .. 2 ) {
        my $Address = $System->insertChild( 'Address', {
            'Alias'   => 'DE_'.$TryNumber,
            'Street'  => 'Leutragraben 1',
            'City'    => 'Jena',
            'Zipcode' => '07740',
            'Company' => 'epages',
            'BankAccountHolder' => 'Max Mustermann',
            'BankAccountNo' => '1234567890',
            'BankCode' => '10020055',
            'BankName' => 'Musterbank',
        });

        # EPG-15717: don't get value from cache (for every second try)
        UpdateLocalConnection( $dbi->cacheKey ) if 0 == $TryNumber % 2;

        # Get methods should decrypt the data transparently.
        is( $Address->get('BankAccountNo'), '1234567890',
            "BankAccountNo $TryNumber" );
        is( $Address->get('BankName'), 'Musterbank', "BankName $TryNumber" );

        # Get the encrypted data from DB directly.
        my $SQL = "SELECT bankaccountno, bankname FROM addresscompany WHERE addressid = ?";
        my $aaRows = GetCurrentDBHandle()->execute( $SQL, [ $Address->id ] );

        is( length $aaRows->[0]->[0], 64, "BankAccountNo encrypted $TryNumber" );
        is( length $aaRows->[0]->[1], 64, "BankName encrypted $TryNumber" );

        # Update test
        # Get methods should decrypt the data transparently.
        $Address->set({'BankAccountNo' => '0987654321'});
        is( $Address->get('BankAccountNo'), '0987654321',
            "BankAccountNo Update");

        $Address->set({'BankName' => 'Muster_Bank'});
        is( $Address->get('BankName'), 'Muster_Bank', "BankName Update" );
    }




    ok( Cleanup(), "cleanup" );
    DisconnectSite();
}

sub Cleanup {
    foreach my $Path (qw (O1 O2 DE_1 DE_2)) {
        if (ExistsObjectByPath($Path)) {
            my $object = LoadObjectByPath($Path);
            $object->delete;
        }
    }
    my $ClassObject = LoadClassByAlias( 'Address' );
    $ClassObject->deleteAttribute('L1');
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Address' );
    $ClassObject->insertAttribute({
        'Alias'  => 'L1',
        'Type'   => 'LocalizedString',
        'Package'=> 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute',
    });
    LoadRootObject()->insertChild( 'Address', {
        'Alias'   => 'O1',
        'Street'  => 'Leutragraben 2-4',
        'City'    => 'Jena',
        'Zipcode' => '07740',
        'Company' => 'epages',
    });
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
