﻿use strict;
use utf8;
use warnings FATAL => 'all';
use Test::More tests => 494;
use DE_EPAGES::Address::API::Attributes::Address_Ext;
use DE_EPAGES::Core::API::Script qw( RunScript );

sub _TestHouseNumber {
    my ($Street, $StreetOnlyExp, $NumberExp) = @_;

    ## no critic qw(ProtectPrivateSubs) # the private function is much faster than the "real" API, but it should remain private
    my( $StreetOnly, $Number ) = DE_EPAGES::Address::API::Attributes::Address_Ext::_ExtractHouseNumber( $Street );
    ## use critic

    $Street //= '(undef)';
    is $StreetOnly, $StreetOnlyExp, "street name from [$Street]";
    is $Number, $NumberExp, "house number from [$Street]";
}

# apply a filter function to $Input; the filter function modifies a local $_
sub _Filter($$) {
    my ($Sub, $Input) = @_;

    local $_ = $Input;
    $Sub->();
    return $_;
}

# apply a filter function to an array of arrays
sub _ModifyData($$) {
    my ($aData, $Sub) = @_;

    return [ map { _Filter( $Sub, $_ ) } @$aData ];
}

# apply a filter function to the test data and run the test with the modified data
sub _TestHouseNumbers(&@) {
    my ($Sub, @Data) = @_;

    _TestHouseNumber @$_ for map { _ModifyData( $_, $Sub ) } @Data;
}

sub Main {

    my @TestData = (
        [ 'Schönberger Straße 20',              => 'Schönberger Straße', '20' ],
        [ 'Schönberger Straße 20a',             => 'Schönberger Straße', '20a' ],
        [ ' Schönberger Straße 1a !  ',         => 'Schönberger Straße', '1a !' ],
        [ "\tSchönberger Straße 1a !  ",        => 'Schönberger Straße', '1a !' ],
        [ "\nSchönberger Straße 1a !  ",        => 'Schönberger Straße', '1a !' ],
        [ "\n\nSchönberger Straße 1a !  ",      => 'Schönberger Straße', '1a !' ],
        [ "\r\n\r\nSchönberger Straße 1a",      => 'Schönberger Straße', '1a' ],
        [ 'Schönberger Straße 20 a',            => 'Schönberger Straße', '20 a' ],
        [ ' Schönberger Straße 20 a  ',         => 'Schönberger Straße', '20 a' ],
        [ ' Schönberger Straße 20 a !  ',       => 'Schönberger Straße', '20 a !' ],
        [ ' Schönberger Straße 20.a !  ',       => 'Schönberger Straße', '20.a !' ],
        [ ' Schönberger Straße 20,a !  ',       => 'Schönberger Straße', '20,a !' ],
        [ ' Schönberger Straße 20,a .  ',       => 'Schönberger Straße', '20,a .' ],
        [ ' Schönberger Straße 20,a.  ',        => 'Schönberger Straße', '20,a.' ],
        [ " Schönberger Straße 20\ta.  ",       => 'Schönberger Straße', '20 a.' ],
        [ "Schönberger Straße\t20\ta.  ",       => 'Schönberger Straße', '20 a.' ],
        [ "Schönberger\tStraße\t 20 \ta.  ",    => 'Schönberger Straße', '20 a.' ],
        [ "Schönberger\tStraße\t 20 \ta.  ",    => 'Schönberger Straße', '20 a.' ],
    );
    _TestHouseNumber @$_ for @TestData;
    _TestHouseNumbers { s/\b20\b/42-44/ } @TestData;
    _TestHouseNumbers { s/Straße/Str./ } @TestData;
    _TestHouseNumbers { s/Straße/Str./; s/\b20\b/42-44/ } @TestData;
    _TestHouseNumbers { s/Straße/straße/ } @TestData;
    _TestHouseNumbers { s/Straße/Weg/ } @TestData;
    _TestHouseNumbers { s/Straße/Weg/; s/\b20\b/42-44/ } @TestData;
    _TestHouseNumbers { s/Straße/W./ } @TestData;
    _TestHouseNumbers { s/Straße/W./; s/\b20\b/42-44/ } @TestData;

    # test French data
    @TestData = (
        [ '13 rue capucines',                   => 'rue capucines', '13' ],
        [ '13, rue capucines',                  => 'rue capucines', '13' ],
        [ '13- rue capucines',                  => 'rue capucines', '13' ],
        [ '13 - rue capucines',                 => 'rue capucines', '13' ],
        [ '13. rue capucines',                  => 'rue capucines', '13' ],
        [ '13: rue capucines',                  => 'rue capucines', '13' ],
        [ '13; rue capucines',                  => 'rue capucines', '13' ],
        [ '13a rue capucines',                  => 'rue capucines', '13a' ],
        [ '13 a rue capucines',                 => 'rue capucines', '13 a' ],
        [ '13.a rue capucines',                 => 'rue capucines', '13.a' ],
        [ '13.a, rue capucines',                => 'rue capucines', '13.a' ],
        [ '13.ab, rue capucines',               => 'rue capucines', '13.ab' ],
    );
    _TestHouseNumber @$_ for @TestData;
    _TestHouseNumbers { s/\brue\b/r./ } @TestData;
    _TestHouseNumbers { s/\brue\b/r/ } @TestData;

    # test housenumber ranges
    @TestData = ( # range indicator used here: plain old ASCII minus (dash)
        [ 'Herdentorsteinweg 44-45',            => 'Herdentorsteinweg', '44-45' ],
        [ 'Herdentorsteinweg 1a-1c',            => 'Herdentorsteinweg', '1a-1c' ],
        [ 'Herdentorsteinweg 1-2',              => 'Herdentorsteinweg', '1-2'   ],
        [ 'Herdentorsteinweg 1 -2',             => 'Herdentorsteinweg', '1 -2'  ],
        [ 'Herdentorsteinweg 1- 2',             => 'Herdentorsteinweg', '1- 2'  ],
        [ 'Herdentorsteinweg 1 - 2',            => 'Herdentorsteinweg', '1 - 2' ],
        [ 'Straße des 17 Juni 106-108'          => 'Straße des 17 Juni', '106-108'],
        [ 'Straße des 17. Juni 106-108'         => 'Straße des 17. Juni', '106-108'],
    );
    _TestHouseNumber @$_ for @TestData;

    # run more tests using fancy new-fangled dashes
    _TestHouseNumbers { s/-/\x{2012}/ } @TestData;
    _TestHouseNumbers { s/-/\x{2013}/ } @TestData;
    _TestHouseNumbers { s/-/\x{2014}/ } @TestData;

    # Mannheim
    @TestData = (
        [ 'N2, 10'        => 'N2', '10' ], # comma
        [ 'N2 10'         => 'N2', '10' ],
        [ 'N 2, 10'       => 'N 2', '10' ],
        [ 'N 2 10'        => 'N 2', '10' ],
    );
    _TestHouseNumber @$_ for @TestData;
    _TestHouseNumbers { s/,/;/ } @TestData; # semicolon
    _TestHouseNumbers { s/,/:/ } @TestData; # colon

    # edge cases
    @TestData = (
        [ '',                                       '', ''],
        [ undef,                                    '', ''],
        [ '99',                                     '99', ''],
        [ 'Gurkenchaussee',                         'Gurkenchaussee', ''],
        [ 'Gurkenchaussee jawoll! -99',             'Gurkenchaussee jawoll!', '-99'],
    );
    _TestHouseNumber @$_ for @TestData;
}

RunScript(Sub=>\&Main, Debug=>1);
