use Test::Simple tests => 26;
use strict;
use Config;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Transformation qw( Transform );
use DE_EPAGES::Core::API::Transformation::ImportHandler qw (ReadTransformation);
use DE_EPAGES::Core::API::Transformation::Color qw( String2Color Hsv2Rgb Rgb2Hsv hardFloor );

##no critic qw(ProhibitMagicNumbers)
sub CompareImage {
    my ($ImageFile1, $ImageFile2) = @_;

    my $Image1 = Image::Magick->new();
    $Image1->Read($ImageFile1);

    my $Image2 = Image::Magick->new();
    $Image2->Read($ImageFile2);

    return undef if ($Image1->get('width') != $Image2->get('width') ||
                     $Image1->get('height') != $Image2->get('height'));

    # disable transparency - problems with gifs and pngs
    $Image1->Set(matte => 'False');
    $Image2->Set(matte => 'False');

    $Image1->Compare($Image2);

    # mean difference per pixel may not vary by more than this value
    # this may be caused by JPEG dithering
    return $Image1->Get('mean-error') <= 0.0002;
}

sub CompareColors {
    my ($Color, $R, $G, $B, $A) = @_;
    my $c = (hardFloor($Color->{'R'} * 255) eq $R &&
              hardFloor($Color->{'G'} * 255) eq $G &&
              hardFloor($Color->{'B'} * 255) eq $B &&
              hardFloor($Color->{'A'} * 255) eq $A);
    unless ($c) {
        print int($Color->{'R'} * 255).',';
        print int($Color->{'G'} * 255).',';
        print int($Color->{'B'} * 255).',';
        print int($Color->{'A'} * 255)."\n";
    }
    return $c;
}

sub Main {
    my ($Result, $Expected);

    # process it
    my $hTransformation = ReadTransformation('./Transformation/Image/transform.xml');
    my $hResult = Transform($hTransformation, {'SourceDirectory' => './Transformation/Image/Source',
        'TargetDirectory' => './Transformation/Image/Source', 'BaseDirectory' => './Transformation/Color'});

    ok( 1, 'Process - did not fail' );

    # test 1
    $Expected = './Transformation/Image/test1.gif';
    $Result = './Transformation/Image/Source/bgr_header.gif';
    ok( CompareImage($Expected, $Result), 'Process - test 1' );

    # test 2
    $Expected = './Transformation/Image/test2.gif';
    $Result = './Transformation/Image/Source/bgr_layout.gif';
    ok( CompareImage($Expected, $Result), 'Process - test 2' );

    # check results
    ok( $hResult->{'ContentListHeadHeadlineColor'} eq '#1DF2A1', 'Process - test result, color' );
    ok( $hResult->{'Output1'} eq '../Image/Source/bgr_layout.gif', 'Process - test result, image' );
    ok( $hResult->{'width'} eq '500px', 'Process - test result, object width' );
    ok( $hResult->{'width2'} eq $hResult->{'width'}, 'Process - test result, object width2' );
    ok( $hResult->{'OutputColor1'} eq '#925C90', 'Process - test result, OverlayColor with object' );
    ok( $hResult->{'OutputColor2'} eq '#FFA8C9', 'Process - test result, OverlayColor with constant' );

    # ~Transformation::Color test cases
    my $Transformation = 'DE_EPAGES::Core::API::Transformation::Color';
    ok(CompareColors(String2Color('rgb(123,234,111)'), 123, 234, 111, 255), 'String2Color - rgb(123,234,111)');
    ok(CompareColors(String2Color('rgb(123,234,111,102)'), 123, 234, 111, 102), 'String2Color - rgb(123,234,111,102)');
    ok(CompareColors(String2Color('rgba(123,234,111,0.6)'), 123, 234, 111, 153), 'String2Color - rgba(123,234,111,0.6)');
    ok(CompareColors(String2Color('red'), 255, 0, 0, 255), 'String2Color - red');
    ok(CompareColors(String2Color('#ABC'), 170, 187, 204, 255), 'String2Color - #ABC');
    ok(CompareColors(String2Color('#043050'), 4, 48, 80, 255), 'String2Color - #043050');
    ok(CompareColors(String2Color('transparent'), 0, 0, 0, 0), 'String2Color - transparent');
    ok(CompareColors(String2Color('none'), 0, 0, 0, 0), 'String2Color - none');
    ok(CompareColors(Hsv2Rgb(Rgb2Hsv({R => 123/255, G => 234/255, B => 111/255, A => 255/255})), 123, 234, 111, 255), 'RGB -> HSV -> RGB: 123,234,111,255');
    ok(CompareColors($Transformation->AddColorOffset({R => 123/255, G => 234/255, B => 111/255, A => 255/255},
                                    {'Hue' => '10', 'Saturation' => '-5', 'Brightness' => '20'}), 133, 255, 142, 255), 'AddColorOffset - 123,234,111,255');
    ok(CompareColors($Transformation->AddColorOffset({R => 85/255, G => 170/255, B => 255/255, A => 255/255},
                                    {'Hue' => '360', 'Saturation' => '100', 'Brightness' => '100'}), 0, 127, 255, 255), 'AddColorOffset - 85,170,255,255 + H360S100B100');
    ok(CompareColors($Transformation->AddColorOffset({R => 85/255, G => 170/255, B => 255/255, A => 255/255},
                                    {'Hue' => '-360', 'Saturation' => '-100', 'Brightness' => '-100'}), 0, 0, 0, 255), 'AddColorOffset - 85,170,255,255 + H-360S-100B-100');
    ok(CompareColors($Transformation->OverlayColor({R => 85/255, G => 170/255, B => 255/255, A => 255/255},
                                    {'Color' => '#FFFFFF', 'Transparency' => '50'}), 170, 212, 255, 255), 'OverlayColor - #FF0000 + #FFFFFF(50%)');
    ok(CompareColors($Transformation->OverlayColor({R => 85/255, G => 170/255, B => 255/255, A => 255/255},
                                    {'Color' => '#FFFFFF', 'Transparency' => '0'}), 85, 170, 255, 255), 'OverlayColor - #FF0000 + #FFFFFF(0%)');
    ok(CompareColors($Transformation->OverlayColor({R => 85/255, G => 170/255, B => 255/255, A => 255/255},
                                    {'Color' => '#FFFFFF', 'Transparency' => '100'}), 255, 255, 255, 255), 'OverlayColor - #FF0000 + #FFFFFF(100%)');
    ok(CompareColors(String2Color('#ABC'), 170, 187, 204, 255), 'String2Color - #ABC');

    ok($hResult->{'OutputColor4'} eq '#D5D5D5', 'AddColorOffset to gray');
}

RunScript(Sub=>\&Main, Debug=>1);
