use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DateTime;

use DE_EPAGES::Envialia::API::RemoteEnvialiaTransaction;

use strict;

my $ROOTPATH = '/Test';


sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                testImport();
                my $ret = testLoginToken();
                testShipmentDelivery($ret) if ($ret);
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}


sub testImport{
    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopEnvialia');
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/EnvialiaLogistics');
    isa_ok($ShippingFulfillmentMethod, 'DE_EPAGES::Envialia::API::Object::ShippingFulfillmentMethodEnvialia');

    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments');

    my $ShippingLabelEnvialia = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    isa_ok($ShippingLabelEnvialia, 'DE_EPAGES::Envialia::API::Object::ShippingLabelEnvialia' );

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderEnvialia');
    isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' );

    cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels');

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        isa_ok($ShippingLabel, 'DE_EPAGES::Envialia::API::Object::ShippingLabelEnvialia' );
    }
}

sub testLoginToken{
    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopEnvialia');
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/EnvialiaLogistics');

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderEnvialia');

    my $RemoteEnvialiaTransaction = DE_EPAGES::Envialia::API::RemoteEnvialiaTransaction->new($ShippingFulfillmentMethod, $Transaction, '2');

    my $LoginRequest = $RemoteEnvialiaTransaction->generateLoginRequest();
    my $ErrorMsg = $RemoteEnvialiaTransaction->callLoginService($LoginRequest);

    # check if login token is returned
    cmp_ok(length( $RemoteEnvialiaTransaction->{'LoginToken'} ),'>',0,'login token returned');
    return 0 if ($ErrorMsg);
    return $RemoteEnvialiaTransaction;
}

sub testShipmentDelivery{
    my ($RemoteEnvialiaTransaction) = @_;

    my $Transaction = $RemoteEnvialiaTransaction->{'Transaction'};
    my $counter = 0;
    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        $counter++;
        #generate an appropriate pick-up date
        $ShippingLabel->set({'PickupDate' => DateTime->now->add('days' => $counter)});

        my $req = $RemoteEnvialiaTransaction->generateRecordingRequest($ShippingLabel);

        my ($trackingID, $trackingLink) = $RemoteEnvialiaTransaction->callGrabaEnvio($req, $ShippingLabel->get('PickupDate'));
        cmp_ok(length($trackingID), '>', 1, 'tracking ID returned');
        cmp_ok(length($trackingLink), '>', 1, 'tracking URL returned');

        if ($trackingID) {
            my $labelRequest = $RemoteEnvialiaTransaction->generateLabelRequest($trackingID);
            my $file = $RemoteEnvialiaTransaction->callConsEtiquetaEnvio($labelRequest);

            cmp_ok(length($file), '>', 1, 'shipping label PDF returned');
        }
    }
}


sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}


sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}



sub Main {
    run_tests;
    done_testing;
}


RunScript( Sub => \&Main );

1;