use Test::More;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath ExistsObjectByPath LoadRootObject);

use DE_EPAGES::MRW::API::RemoteMRWBarcode;

use strict;

my $ROOTPATH = '/Test';

sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                testImport();
                testBarCodeCreation();
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub testImport{
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShopMRW');
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/MRW');
    isa_ok($ShippingFulfillmentMethod, 'DE_EPAGES::MRW::API::Object::ShippingFulfillmentMethodMRW');

    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments');

    my $ShippingLabelMRW = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    isa_ok($ShippingLabelMRW, 'DE_EPAGES::MRW::API::Object::ShippingLabelMRW' );

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderMRW');
    isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' );

    cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels');

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        isa_ok($ShippingLabel, 'DE_EPAGES::MRW::API::Object::ShippingLabelMRW' );
    }
}

sub testBarCodeCreation {
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShopMRW');
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/MRW');
    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderMRW');

    my $Remote = DE_EPAGES::MRW::API::RemoteMRWBarcode->new();
    my $cont = 0;
    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')})
    {   
        $cont++; 
        my $result = $Remote->processShipmentRequest($ShippingFulfillmentMethod,$ShippingLabel,'GIF',1);
        cmp_ok(isValid( $result->{'Label'} ),'==',1,'label is returned');
    }
    cmp_ok($cont, '==',2, 'number of returned items');
}


sub Main {
    run_tests;
    done_testing;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

1;