use strict;
use Test::More;

use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MultiStore::API::CloneMasterShop qw ( CloneMasterShop CreateCacheShop RegisterCacheShopAtSubShop );
use DE_EPAGES::MultiStore::API::Object::PagerVisibleProducts;
use DE_EPAGES::MultiStore::API::SyncSubShop qw ( SyncShop );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath JoinObjectPath LoadObjectByPath LoadRootObject );

my $ROOTPATH = 'Test';
my $ABSOLUTE_ROOTPATH = JoinObjectPath( [ '', $ROOTPATH ] );

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            unless ( ExistsCartridgeByPackage('DE_EPAGES::MultiStore') ) {
                plan skip_all => 'MultiStore cartridge not installed';
            }
            unless ( ExistsCartridgeByPackage('DE_EPAGES::MultiStoreMaster') ) {
                plan skip_all => 'MultiStoreMaster cartridge not installed';
            }

            ok( Init(), 'init' );

            # tests
            eval { RunTests(); };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub RunTests {
    TestCreateCacheShop();
    TestSyncShop();
}

sub TestSyncShop {
    my $SubShop   = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestSubShop' ] ) );
    my $CacheShop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestCacheShop' ] ) );

    eval { $CacheShop->path('Categories/Tents')->cloneToSubShop( $SubShop->path('Categories') ) };
    if ( ExistsError() ) {
        fail( 'Error: ' . GetError()->longMessage );
        return;
    }

    #Copy categories and products
    SyncShop( $SubShop->get('Path') );
    is(
        ExistsObjectByPath( join( '/', $CacheShop->get('Path'), 'Categories/Tents/FamilyTents' ) ) == 1,
        ExistsObjectByPath( join( '/', $SubShop->get('Path'), 'Categories/Tents/FamilyTents' ) ) == 1,
        'Copy Category'
    );

    is(
        ExistsObjectByPath( join( '/', $CacheShop->get('Path'), 'Products/wb_1003111010' ) ) == 1,
        ExistsObjectByPath( join( '/', $SubShop->get('Path'), 'Products/wb_1003111010' ) ) == 1,
        'Copy Product'
    );


    #Delete categories and products
    $CacheShop->path('Categories/Tents/FamilyTents')->delete,
    $CacheShop->path('Products/wb_1003111010')->delete;
    SyncShop( $SubShop->get('Path') );

    is(
        ExistsObjectByPath( join( '/', $CacheShop->get('Path'), 'Categories/Tents/FamilyTents' ) ) == 0,
        ExistsObjectByPath( join( '/', $SubShop->get('Path'), 'Categories/Tents/FamilyTents' ) ) == 0,
        'Delete Category'
    );

    is(
        ExistsObjectByPath( join( '/', $CacheShop->get('Path'), 'Products/wb_1003111010' ) ) == 0,
        ExistsObjectByPath( join( '/', $SubShop->get('Path'), 'Products/wb_1003111010' ) ) == 0,
        'Delete Product'
    );

    return;
}

sub TestCreateCacheShop {
    my $SubShop = LoadObjectByPath( JoinObjectPath( [ '', $ROOTPATH, 'TestSubShop' ] ) );
    my $CacheShop = CreateCacheShop( {
            'MasterStore' => GetTestSite(),
            'MasterShop'  => $ABSOLUTE_ROOTPATH . '/TestMasterShop',
            'CacheShop'   => $ABSOLUTE_ROOTPATH . '/TestCacheShop',
        },
        0
    );
    pass('create cache shop');

    CloneMasterShop($CacheShop);
    pass('clone master shop');

    RegisterCacheShopAtSubShop( $CacheShop, $SubShop );
    is( $SubShop->get('CacheShop')->id, $CacheShop->id, 'registered cache shop.' );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    BlockWarnings( sub {
        eval { LoadObjectByPath($ABSOLUTE_ROOTPATH)->delete if ExistsObjectByPath($ABSOLUTE_ROOTPATH); };
        Warning( GetError() ) if ( ExistsError() );
    });
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ABSOLUTE_ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => $ROOTPATH } );
    ImportObjects( 'MasterShopComplete.xml', $ABSOLUTE_ROOTPATH );
    ImportObjects( 'SubShopEmpty.xml',       $ABSOLUTE_ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
