#!perl -w
use strict;
use Test::More tests => 38;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw (IsA GetCurrentDateTime GetLocalTimeZone AddDuration SubtractDuration);
use DE_EPAGES::Database::API::Connection qw (ConnectTestSite DisconnectSite);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject LoadClassByAlias ExistsClassByAlias);

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::IniConfigAttribute';
note "$ATTRIBUTE_PACKAGE\n";
RunScript(Sub=>\&Main, Debug=>1);

$ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';
note "$ATTRIBUTE_PACKAGE\n";
RunScript(Sub=>\&Main, Debug=>1);

sub Main {
    ConnectTestSite();
    # create test classes
    ok( Init(), 'initialization ' . $ATTRIBUTE_PACKAGE );
    #'FileName' =>'IniConfigAttribute.conf',
    my $Object = LoadRootObject->insertChild( 'C1', { Alias => 'C1O1' } );

    my $Date = GetCurrentDateTime();
    ok($Date->time_zone()->name(), 'tz '.$Date->time_zone()->name() );

    my $SiteLocale = $Object->getSite->siteLocale('en_US');
    like( $SiteLocale->formatDateTime($Date, 'datetime', 'mnemonic'), qr/^today/, 'formatDateTime(mnemonic-today)' );

    SubtractDuration($Date, 'days', 1);
    like( $SiteLocale->formatDateTime($Date, 'datetime', 'mnemonic'), qr/^yesterday/, 'formatDateTime(mnemonic-yesterday)' );

    $Date = AddDuration(GetCurrentDateTime(), 'months', 1);
    like( $SiteLocale->formatDateTime($Date, 'datetime', 'mnemonic'), qr/^\d+/, 'formatDateTime(mnemonic-next month)' );

    $Date = AddDuration( AddDuration(GetCurrentDateTime(), 'days', 1), 'months', 1 );
    like( $SiteLocale->formatDateTime($Date, 'datetime', 'mnemonic'), qr/^\d+/, 'formatDateTime(mnemonic-next month + 1 day)' );

    $Date = DateTime->new( year => 1964, month => 10, day => 16,
        hour => 16, minute => 12, second => 47,
        time_zone => 'Asia/Taipei',
    );

    $Object->set({
        'TestDate' => $Date->clone,
        'TestDateCached' => $Date->clone,
    });
    my $CompareDate = $Object->get('TestDate');
    my $CompareDateCached = $Object->get('TestDateCached');

    ok( IsA( $CompareDate, 'DateTime' ), 'attribute value is DateTime' );
    ok( IsA( $CompareDateCached, 'DateTime' ), 'attribute value is DateTime' );

    #print $CompareDate->time_zone->name, "\n";
    #print $CompareDateCached->time_zone->name, "\n";

    foreach ($CompareDate, $CompareDateCached) {
        my $StrValue = $SiteLocale->formatDateTime($_, 'datetime', 'medium');
        is( $StrValue, 'Oct 16, 1964 9:12:47 AM', 'formatDateTime' );
    }

    $Date->set_time_zone( GetLocalTimeZone() );
    $CompareDate->set_time_zone( GetLocalTimeZone() );
    $CompareDateCached->set_time_zone( GetLocalTimeZone() );

    cmp_ok($CompareDate, 'eq', $Date, 'get = set');
    cmp_ok($CompareDateCached, 'eq', $Date, 'get = set');

    $CompareDate->set_time_zone( GetLocalTimeZone() );
    cmp_ok($CompareDate, 'eq', $Date, 'still the same time after converting time zone to server time' );

    $CompareDate->set_time_zone( 'America/Chicago' );
    cmp_ok($CompareDate, 'eq', $Date, 'still the same time after converting time zone to other time' );

    $Date->set_time_zone( GetLocalTimeZone() ) if $Date->time_zone()->is_floating;
    $Date->set_time_zone( 'America/Chicago' );

    $Object->set({'TestDate' => $Date->clone});
    $CompareDate = $Object->get('TestDate');

    $CompareDate->set_time_zone( GetLocalTimeZone() ) if $CompareDate->time_zone()->is_floating;
    $CompareDate->set_time_zone( 'America/Chicago' );

    ok($CompareDate eq $Date, 'date '. "${Date} - $CompareDate");
    ok($CompareDate->time_zone()->name() eq $Date->time_zone()->name(), 'tz '.$Date->time_zone()->name().':'.$CompareDate->time_zone->name() );

    $Object->set({
        'TestDate' => undef,
        'TestString' => undef,
    });
    ok( !defined $Object->get('TestDate'), 'set DateTime attribute to undef' );
    ok( !defined $Object->get('TestString'), 'set String attribute to undef' );

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C1 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'TestDate', Type => 'DateTime', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'TestDateCached', Type => 'DateTime', IsCacheable => 1, Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'TestString', Type => 'String', Package => $ATTRIBUTE_PACKAGE });

    return 1;
}
