use strict;
use English;
use Config;
use Test::More tests => 19;
use Text::Diff qw( diff );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw( DeleteDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );
use DE_EPAGES::Database::API::Connection qw( GetTestSite );
use DE_EPAGES::Object::API::Factory qw( JoinObjectPath );

my $Provider = 'Distributor';
my $User = 'admin';
my $Password = 'admin';
my $Alias = 'TransferShop';
my $TempAlias = 'OldTransferShop';
my $ProductAlias = 'lt_0401107001';
my $StoreName = $ENV{EPAGES_TESTSITE} || GetTestSite();
my $ShopType = 'Demo';
my $PERL = $EXECUTABLE_NAME;
my $DE_EPAGES = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES';
my $TempDir = $Config{'osname'} eq 'MSWin32' ? $ENV{'TEMP'} // $ENV{'TMP'} : '/tmp';
my $DEBUG = 0;

sub Init {
    CleanUp();
    ok( 1, 'init' );
}

sub CleanUp {
    DeleteDirectory( 'out', { recursive => 1 } );

    SystemCommand([ $PERL, "$DE_EPAGES/ShopConfiguration/Scripts/deleteShop.pl",
        -wsuser => JoinObjectPath(['', 'Providers', $Provider, 'Users', $User]),
        -passwd => $Password,
        -alias => $Alias,
    ], { Print => $DEBUG });
    SystemCommand([ $PERL, "$DE_EPAGES/ShopConfiguration/Scripts/deleteShop.pl",
        -wsuser => JoinObjectPath(['', 'Providers', $Provider, 'Users', $User]),
        -passwd => $Password,
        -alias => $TempAlias,
    ], { Print => $DEBUG });
    ok( 1, 'cleanup' );
}

sub TestD {

    SystemCommand([ $PERL, "$DE_EPAGES/ShopConfiguration/Scripts/createShop.pl",
        -wsuser => JoinObjectPath(['', 'Providers', $Provider, 'Users', $User]),
        -passwd => $Password,
        -alias => $Alias,
        -scriptname => $Alias,
        -shoptype => $ShopType,
        -storename => $StoreName,
    ], { Print => $DEBUG });
    ok( 1, 'create shop' );

    # EPG-19620: ShopTransfer failed with AnonymousUsers: create 2 baskets
    my $HostName = `$PERL $DE_EPAGES/Object/Scripts/get.pl -storename $StoreName -path / DomainName`;
    $HostName =~ s/^DomainName=//;
    $HostName =~ s/\s*$//s;
    for( 1..2 ) {
        my $ProductPath = JoinObjectPath(['', 'Shops', $Alias, 'Products', $ProductAlias]);
        SystemCommand([ $PERL, "$DE_EPAGES/Core/Scripts/WebGet.pl",
            -url => "http://$HostName/epages/$Alias.sf/en_GB/?ObjectPath=$ProductPath&ChangeAction=AddToBasket",
            -file => "$TempDir/addtobasket.html",
        ], { Print => $DEBUG });
        ok( 1, 'create baskets as anonymous user' );
    }

    SystemCommand([ $PERL, "$DE_EPAGES/Object/Scripts/export.pl",
        -storename => $StoreName,
        -path => '/Shops',
        -file => "$TempDir/source_shop_exported.xml",
        qw( -withdefaults -withoutguids -withoutcreationdate),
        $Alias
    ], { Print => $DEBUG });
    ok( 1, 'export transfered shop for comparison' );

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/setClosed.pl",
        -provider => $Provider,
        -shopid => $Alias,
    ], { Print => $DEBUG });
    ok( 1, 'close shop' );

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/export.pl",
        -storename => $StoreName,
        -shop => $Alias,
        $DEBUG ? ( '-verbose' ) : ()
    ], { Print => $DEBUG });
    ok( 1, 'export shop' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/renameShopAtSite.pl",
        -storename => $StoreName,
        -shop => $Alias,
        -newalias => $TempAlias,
        $DEBUG ? ( '-verbose' ) : ()
    ], { Print => $DEBUG });
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/renameShopAtStore.pl",
        -storename => $StoreName,
        -shop => $Alias,
        -newalias => $TempAlias,
        $DEBUG ? ( '-verbose' ) : ()
    ], { Print => $DEBUG });
    ok( 1, 'rename shop' );

    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/export.pl",
        -storename => $StoreName,
        '-system',
        $DEBUG ? ( '-verbose' ) : ()
    ], { Print => $DEBUG });
    ok( 1, 'export system' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/transform.pl",
        -storename => $StoreName,
        -source => $StoreName,
        -shop => $Alias,
        '-newguid',
        $DEBUG ? ( '-verbose' ) : ()
    ], { Print => $DEBUG });
    ok( 1, 'transform shop' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/import.pl",
        -storename => $StoreName,
        -shop => $Alias,
        $DEBUG ? ( '-verbose' ) : ()
    ], { Print => $DEBUG });
    ok( 1, 'import shop' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/createShopRef.pl",
        -storename => $StoreName,
        -shop => $Alias,
    ], { Print => $DEBUG });
    ok( 1, 'createShopRef' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/insertPoolDB.pl",
        -storename => $StoreName,
        -shop => $Alias,
    ], { Print => $DEBUG });
    ok( 1, 'insertPoolDB' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/setVariationStrings.pl",
        -storename => $StoreName,
        -shop => $Alias,
    ], { Print => $DEBUG });
    ok( 1, 'setVariationStrings' );
    SystemCommand([ $PERL, "$DE_EPAGES/ShopTransfer/Scripts/setClosedByProvider.pl",
        -storename => $StoreName,
        -shop => $Alias,
        '-open',
    ], { Print => $DEBUG });
    ok( 1, 'open shop' );

    SystemCommand([ $PERL, "$DE_EPAGES/Object/Scripts/export.pl",
        -storename => $StoreName,
        -path => '/Shops',
        -file => "$TempDir/transfered_shop_exported.xml",
        qw( -withdefaults -withoutguids -withoutcreationdate),
        $Alias
    ], { Print => $DEBUG });
    ok( 1, 'export transfered shop for comparison' );

    # Comparing old shop and transfered shop.
    my $diff = diff "$TempDir/source_shop_exported.xml", "$TempDir/transfered_shop_exported.xml";
    is( $diff, '', 'no difference' );
}

RunScript( Message => 'long', Sub => sub {
    Init();
    eval {
        TestD();
    };
    if( ExistsError() ) {
        print STDERR GetError()->longMessage;
    }
    CleanUp();
});
