use strict;
use Test::More tests => 4;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObjectByPath );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Core::API::Transformation::ImportHandler qw ( ReadTransformation );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf GetCurrentDateTime );
use DE_EPAGES::Core::API::Transformation qw ( Transform );
use DE_EPAGES::Design::API::Style qw ( AddStyle );
use Time::HiRes qw( gettimeofday tv_interval );

Readonly::Scalar my $MAX_TRANSFORMATION_TIME => 6.5; # Baselines: 6.13.3: 6.5.

# Creates a shop and looks, if the style exists.
RunScript(Sub => sub {

    ConnectTestSite();
    my $System = LoadRootObject();
    ok (defined $System, 'system');
    my $Folder = $System->folder('Shops');

    # Insert new testshop.
    $Folder->child('TestShop')->delete if $Folder->existsChild('TestShop');
    my $Shop = $Folder->insertChild('Shop', {
        'Alias'=>'TestShop',
    });


    # Set transformation parameter.
    my $Parameters = {
      WizardColor1 => '#FF0000',
      WizardColor2 => '#00FF00',
      WizardColor3 => '#0000FF',
      WizardImage1 => 'Transformation/WizardImage1.jpg',
      WizardImage2 => 'Transformation/WizardImage2.jpg'
    };

    foreach my $StyleName (qw(Boxes Fold SeaWorld)){

         # Instert new style (Standardstyle).
        my $StyleTemplate = LoadObjectByPath('/StyleTemplates/')->child($StyleName);
        #override/add Style
        my $Style = AddStyle($Shop, $StyleTemplate);
        my $StylePublicPath= $Style->get('PublicPath');

        # Get Paths, start timer and read transformation xml.
        my $ImageSetDir = $System->get('PublicStoreTypePath').'/'.$Style->get('LayoutImageSet');
        my $hVars = {'SourceDirectory' => "$ImageSetDir/Source",
                     'TargetDirectory' => $StylePublicPath,
                     'BaseDirectory' => $StylePublicPath};
        my $XmlFile = "$ImageSetDir/transform.xml";
        my $t0 = [gettimeofday];
        my $hTransformation = ReadTransformation($XmlFile);
        my $hObjects = $hTransformation->{'Objects'};

        # mixin attributes from form or (when not submitted) from object
        my @MixinAttributes = qw(WizardColor1 WizardColor2 WizardColor3 WizardImage1 WizardImage2 WizardImage3);
        my @MixinImages = qw(WizardImage1 WizardImage2 WizardImage3);
        my @ChangedAttributes = ();
        foreach my $Attribute (grep { exists($hObjects->{$_}) } @MixinAttributes) {
            my $Value = $Parameters->{$Attribute};

            # add correct path to images
            if (defined $Value && ElementOf($Attribute, @MixinImages)) {
                my $SrcVal;
                if($Value =~ /^MediaGallery/) {
                    $SrcVal = $Style->getSite()->get('PublicPath') . "/$Value";
                } elsif($Value =~ /^EP_Images\//) {
                    $SrcVal = $System->get('ImageDatabasePublicPath')  . "/original/$Value";
                } else {
                    $SrcVal = $StylePublicPath . "/$Value";
                }

                $hObjects->{$Attribute}->{'TargetValue'} = $SrcVal;
                $hObjects->{$Attribute}->{'SourceValue'} = $SrcVal;
            }
            $hObjects->{$Attribute}->{'Value'} = $Value if defined $Value;
        }
        my $hResult = Transform($hTransformation, $hVars, \@ChangedAttributes);

        # Stop timer.
        my $t1= [gettimeofday];

        # set all known object attributes from hash
        my $FilePath = $System->get('PublicPath').'/'.$Style->get('FilePath');
        my $hSetAttributes = {map{$_ => $hResult->{$_}} grep{$Style->class->existsAttribute($_)} keys %$hResult};
        foreach my $attr (keys %$hSetAttributes) {
            $hSetAttributes->{$attr} =~ s/^\Q $FilePath \E[\\\/]*//x if ($attr =~ /Image$/);
        }
        my $date = GetCurrentDateTime();
        $hSetAttributes->{'ChangeDate'} = $date;
        $hSetAttributes->{'ChangedByWizard'} = $date;
        $Style->set($hSetAttributes);

        my $elapsed = tv_interval ($t0, $t1);
        ok ($elapsed < $MAX_TRANSFORMATION_TIME, sprintf( "Transformation takes less than $MAX_TRANSFORMATION_TIME seconds. This run took %0.2f seconds.", $elapsed) );
    }
    $Shop->delete;

    DisconnectSite();
}, Debug=>1);
