use strict;
use Test::Simple tests => 81;

use DBI;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::OrderFile qw( GetSqlFileOrder );
use DE_EPAGES::Core::API::File qw( GetFileContent GetFileContentLines AbsolutePath DeleteFile);
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::Core::API::Error qw( ExistsError );
use Data::Dumper;

sub Prepare {

    DeleteFile('Test.db');

    #connect to patch system info db, create if needed
    my $dbh = DBI->connect(
        "dbi:SQLite:dbname=Test.db", #DSN: dbi, driver, database file
        '',                          #no user
        '',                          #no password
        { RaiseError => 1 },         #complain if something goes wrong
    ) or die $DBI::errstr;

    my $SQLiteDBPath = AbsolutePath('../Database/SQLite');

    #get sql files in right order
    my $aFileNames = my $aFiles = GetSqlFileOrder( $SQLiteDBPath );

    #for each filename process sql statements
    foreach my $FileName (@$aFileNames){

        my $rContent = GetFileContent( $FileName );

        #foreach sql statement in file
        foreach my $SQLStatement ( grep { /\S/ } split /\ngo\b/i, $$rContent ) {

            #do statement or die
            $dbh->do($SQLStatement)
                  or die $DBI::errstr;
        }
    }

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('1','1.1.1')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('2','1.1.2')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('3','1.1.3')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('4','1.2.1')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('5','1.2.2')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('6','1.2.3')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('7','2.1.1')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('8','2.1.2')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('9','2.1.3')")
            or die $DBI::errstr;

    $dbh->do("INSERT INTO patchsystemattribute (key,value) VALUES ('ePagesTargetVersion','2.1.3')")
            or die $DBI::errstr;
}

sub Main {

    local $| = 1;

    my $Help;

    GetOptions(
        'help'          => \$Help,
    );

    usage() if $Help;

    #test preperation
    Prepare();

    #get patchsysteminfohandler module file name
    my $PatchSystemInfoHandlerModul = AbsolutePath('../API/Patch/PatchSystemInfoHandler.pm');

    #use file
    do $PatchSystemInfoHandlerModul; die $@ if $@;

    #test 01 - exists patch system info
    my $PatchSystemInfoHandler = API::Patch::PatchSystemInfoHandler->new('Test.db');

    $PatchSystemInfoHandler->setStartTime();
    my $StartTime = $PatchSystemInfoHandler->getStartTime();

    ok($PatchSystemInfoHandler->existsBasicAttribute('ePagesTargetVersion'), 'key \'ePagesTargetVersion\' exists');

    ok(not ($PatchSystemInfoHandler->existsBasicAttribute('EpagesSharedDir')), 'key \'EpagesSharedDir\' does not exists');

    eval{
        ok(not $PatchSystemInfoHandler->existsBasicAttribute('AnyKey'), 'key \'AnyKey\' does not exists');
    };
    ok( ExistsError(), 'searching for key \'AnyKey\' throws error');


    ok($PatchSystemInfoHandler->getBasicAttribute('ePagesTargetVersion') eq '2.1.3', 'key \'ePagesTargetVersion\' is correct');

    ok($PatchSystemInfoHandler->getPatchTag() eq 'patch_2_1_3', 'patch tag \'patch_2_12_3\' is correct');

    $PatchSystemInfoHandler->setBasicAttribute('ePagesTargetVersion', '2.1.2');
    ok($PatchSystemInfoHandler->getBasicAttribute('ePagesTargetVersion') eq '2.1.2', 'key \'ePagesTargetVersion\' is correctly changed');

    ok($PatchSystemInfoHandler->getPatchTag() eq 'patch_2_1_2', 'patch tag \'patch_2_12_2\' is correct');


    my $TestPath = ['1.1.1','1.1.2','1.1.3','1.2.1','1.2.2','1.2.3','2.1.1','2.1.2','2.1.3'];


    ok (Dumper($PatchSystemInfoHandler->validVersions()) eq Dumper($TestPath), 'version path correct');

    ok ($PatchSystemInfoHandler->validVersion('1.2.2'), 'version \'1.2.2\' is valid');

    ok ( not ($PatchSystemInfoHandler->validVersion('5.5.5')), 'version \'5.5.5\' is not valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch'), 'function \'API::Patch::PatchPrepareHandler::doPatch\' is valid');

    ok ( not ($PatchSystemInfoHandler->validFunction('API::Patch::PatchPopelHandler::doPatch')), 'function \'API::Patch::PatchPopelHandler::doPatch\' is not valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_universal'), 'function \'API::Patch::PatchPrepareHandler::doPatch_universal\' is valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_1_2_2'), 'function \'API::Patch::PatchPrepareHandler::doPatch_1_2_2\' is valid');

    ok ( not ($PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_5_5_5')), 'function \'API::Patch::PatchPrepareHandler::doPatch_5_5_5\' is not valid');

    my $GroundFunctions = [
        'callSubHandlerWithVersionSources',
        'doPatch',
        'doPatch_1_1_1',
        'doPatch_1_1_2',
        'doPatch_1_1_3',
        'doPatch_1_2_1',
        'doPatch_1_2_2',
        'doPatch_1_2_3',
        'doPatch_2_1_1',
        'doPatch_2_1_2',
        'doPatch_2_1_3',
        'doPatch_basic',
        'doPatch_checkPreCondition',
        'doPatch_universal',
        'toExecute',
        'useNow'
    ];

    ok (Dumper($PatchSystemInfoHandler->validGroundFunctions()) eq Dumper($GroundFunctions), 'ground functions correct');

    my $Functions = [
      'API::Patch::PatchPrepareHandler::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandler::doPatch',
      'API::Patch::PatchPrepareHandler::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandler::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandler::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandler::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandler::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandler::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandler::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandler::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandler::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandler::doPatch_basic',
      'API::Patch::PatchPrepareHandler::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandler::doPatch_universal',
      'API::Patch::PatchPrepareHandler::toExecute',
      'API::Patch::PatchPrepareHandler::useNow',
      'API::Patch::PatchPrepareHandlerVersionFolders::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_universal',
      'API::Patch::PatchPrepareHandlerVersionFolders::toExecute',
      'API::Patch::PatchPrepareHandlerVersionFolders::useNow',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::toExecute',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::useNow',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_basic',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_universal',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::toExecute',
      'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::useNow',
      'API::Patch::PatchPrepareHandlerStoreTypes::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_basic',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_universal',
      'API::Patch::PatchPrepareHandlerStoreTypes::toExecute',
      'API::Patch::PatchPrepareHandlerStoreTypes::useNow',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::toExecute',
      'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::useNow',
      'API::Patch::PatchPrepareHandlerDojo::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerDojo::doPatch',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_basic',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerDojo::doPatch_universal',
      'API::Patch::PatchPrepareHandlerDojo::toExecute',
      'API::Patch::PatchPrepareHandlerDojo::useNow',
      'API::Patch::PatchPrepareHandlerJavaScript::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_basic',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerJavaScript::doPatch_universal',
      'API::Patch::PatchPrepareHandlerJavaScript::toExecute',
      'API::Patch::PatchPrepareHandlerJavaScript::useNow',
      'API::Patch::PatchPrepareHandlerCSS::callSubHandlerWithVersionSources',
      'API::Patch::PatchPrepareHandlerCSS::doPatch',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_1_1_1',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_1_1_2',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_1_1_3',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_1_2_1',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_1_2_2',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_1_2_3',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_2_1_1',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_2_1_2',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_2_1_3',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_basic',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_checkPreCondition',
      'API::Patch::PatchPrepareHandlerCSS::doPatch_universal',
      'API::Patch::PatchPrepareHandlerCSS::toExecute',
      'API::Patch::PatchPrepareHandlerCSS::useNow',
      'API::Patch::PatchPreInstallHandlerWebServer::callSubHandlerWithVersionSources',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_1_1',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_1_2',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_1_3',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_2_1',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_2_2',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_1_2_3',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_2_1_1',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_2_1_2',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_2_1_3',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_basic',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_checkPreCondition',
      'API::Patch::PatchPreInstallHandlerWebServer::doPatch_universal',
      'API::Patch::PatchPreInstallHandlerWebServer::toExecute',
      'API::Patch::PatchPreInstallHandlerWebServer::useNow',
      'API::Patch::PatchPreInstallHandlerBackup::callSubHandlerWithVersionSources',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_1_1',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_1_2',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_1_3',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_2_1',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_2_2',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_1_2_3',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_2_1_1',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_2_1_2',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_2_1_3',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_basic',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_checkPreCondition',
      'API::Patch::PatchPreInstallHandlerBackup::doPatch_universal',
      'API::Patch::PatchPreInstallHandlerBackup::toExecute',
      'API::Patch::PatchPreInstallHandlerBackup::useNow',
      'API::Patch::PatchPreInstallHandlerFiles::callSubHandlerWithVersionSources',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_1_1',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_1_2',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_1_3',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_2_1',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_2_2',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_1_2_3',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_2_1_1',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_2_1_2',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_2_1_3',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_basic',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_checkPreCondition',
      'API::Patch::PatchPreInstallHandlerFiles::doPatch_universal',
      'API::Patch::PatchPreInstallHandlerFiles::toExecute',
      'API::Patch::PatchPreInstallHandlerFiles::useNow',
      'API::Patch::PatchInstallHandler::callSubHandlerWithVersionSources',
      'API::Patch::PatchInstallHandler::doPatch',
      'API::Patch::PatchInstallHandler::doPatch_1_1_1',
      'API::Patch::PatchInstallHandler::doPatch_1_1_2',
      'API::Patch::PatchInstallHandler::doPatch_1_1_3',
      'API::Patch::PatchInstallHandler::doPatch_1_2_1',
      'API::Patch::PatchInstallHandler::doPatch_1_2_2',
      'API::Patch::PatchInstallHandler::doPatch_1_2_3',
      'API::Patch::PatchInstallHandler::doPatch_2_1_1',
      'API::Patch::PatchInstallHandler::doPatch_2_1_2',
      'API::Patch::PatchInstallHandler::doPatch_2_1_3',
      'API::Patch::PatchInstallHandler::doPatch_basic',
      'API::Patch::PatchInstallHandler::doPatch_checkPreCondition',
      'API::Patch::PatchInstallHandler::doPatch_universal',
      'API::Patch::PatchInstallHandler::toExecute',
      'API::Patch::PatchInstallHandler::useNow',
      'API::Patch::PatchInstallHandlerServices::callSubHandlerWithVersionSources',
      'API::Patch::PatchInstallHandlerServices::doPatch',
      'API::Patch::PatchInstallHandlerServices::doPatch_1_1_1',
      'API::Patch::PatchInstallHandlerServices::doPatch_1_1_2',
      'API::Patch::PatchInstallHandlerServices::doPatch_1_1_3',
      'API::Patch::PatchInstallHandlerServices::doPatch_1_2_1',
      'API::Patch::PatchInstallHandlerServices::doPatch_1_2_2',
      'API::Patch::PatchInstallHandlerServices::doPatch_1_2_3',
      'API::Patch::PatchInstallHandlerServices::doPatch_2_1_1',
      'API::Patch::PatchInstallHandlerServices::doPatch_2_1_2',
      'API::Patch::PatchInstallHandlerServices::doPatch_2_1_3',
      'API::Patch::PatchInstallHandlerServices::doPatch_basic',
      'API::Patch::PatchInstallHandlerServices::doPatch_checkPreCondition',
      'API::Patch::PatchInstallHandlerServices::doPatch_universal',
      'API::Patch::PatchInstallHandlerServices::toExecute',
      'API::Patch::PatchInstallHandlerServices::useNow',
      'API::Patch::PatchInstallHandlerStores::callSubHandlerWithVersionSources',
      'API::Patch::PatchInstallHandlerStores::doPatch',
      'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
      'API::Patch::PatchInstallHandlerStores::doPatch_1_1_2',
      'API::Patch::PatchInstallHandlerStores::doPatch_1_1_3',
      'API::Patch::PatchInstallHandlerStores::doPatch_1_2_1',
      'API::Patch::PatchInstallHandlerStores::doPatch_1_2_2',
      'API::Patch::PatchInstallHandlerStores::doPatch_1_2_3',
      'API::Patch::PatchInstallHandlerStores::doPatch_2_1_1',
      'API::Patch::PatchInstallHandlerStores::doPatch_2_1_2',
      'API::Patch::PatchInstallHandlerStores::doPatch_2_1_3',
      'API::Patch::PatchInstallHandlerStores::doPatch_basic',
      'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
      'API::Patch::PatchInstallHandlerStores::doPatch_universal',
      'API::Patch::PatchInstallHandlerStores::toExecute',
      'API::Patch::PatchInstallHandlerStores::useNow',
      'API::Patch::PatchInstallHandlerFiles::callSubHandlerWithVersionSources',
      'API::Patch::PatchInstallHandlerFiles::doPatch',
      'API::Patch::PatchInstallHandlerFiles::doPatch_1_1_1',
      'API::Patch::PatchInstallHandlerFiles::doPatch_1_1_2',
      'API::Patch::PatchInstallHandlerFiles::doPatch_1_1_3',
      'API::Patch::PatchInstallHandlerFiles::doPatch_1_2_1',
      'API::Patch::PatchInstallHandlerFiles::doPatch_1_2_2',
      'API::Patch::PatchInstallHandlerFiles::doPatch_1_2_3',
      'API::Patch::PatchInstallHandlerFiles::doPatch_2_1_1',
      'API::Patch::PatchInstallHandlerFiles::doPatch_2_1_2',
      'API::Patch::PatchInstallHandlerFiles::doPatch_2_1_3',
      'API::Patch::PatchInstallHandlerFiles::doPatch_basic',
      'API::Patch::PatchInstallHandlerFiles::doPatch_checkPreCondition',
      'API::Patch::PatchInstallHandlerFiles::doPatch_universal',
      'API::Patch::PatchInstallHandlerFiles::toExecute',
      'API::Patch::PatchInstallHandlerFiles::useNow',
      'API::Patch::PatchInstallHandlerConfigs::callSubHandlerWithVersionSources',
      'API::Patch::PatchInstallHandlerConfigs::doPatch',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_1_1_1',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_1_1_2',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_1_1_3',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_1_2_1',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_1_2_2',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_1_2_3',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_2_1_1',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_2_1_2',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_2_1_3',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_basic',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_checkPreCondition',
      'API::Patch::PatchInstallHandlerConfigs::doPatch_universal',
      'API::Patch::PatchInstallHandlerConfigs::toExecute',
      'API::Patch::PatchInstallHandlerConfigs::useNow',
      'API::Patch::PatchInstallHandlerCartridges::callSubHandlerWithVersionSources',
      'API::Patch::PatchInstallHandlerCartridges::doPatch',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_1',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_2',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_3',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_1_2_1',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_1_2_2',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_1_2_3',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_2_1_1',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_2_1_2',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_2_1_3',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_basic',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_checkPreCondition',
      'API::Patch::PatchInstallHandlerCartridges::doPatch_universal',
      'API::Patch::PatchInstallHandlerCartridges::toExecute',
      'API::Patch::PatchInstallHandlerCartridges::useNow',
      'API::Patch::PatchPostInstallHandler::callSubHandlerWithVersionSources',
      'API::Patch::PatchPostInstallHandler::doPatch',
      'API::Patch::PatchPostInstallHandler::doPatch_1_1_1',
      'API::Patch::PatchPostInstallHandler::doPatch_1_1_2',
      'API::Patch::PatchPostInstallHandler::doPatch_1_1_3',
      'API::Patch::PatchPostInstallHandler::doPatch_1_2_1',
      'API::Patch::PatchPostInstallHandler::doPatch_1_2_2',
      'API::Patch::PatchPostInstallHandler::doPatch_1_2_3',
      'API::Patch::PatchPostInstallHandler::doPatch_2_1_1',
      'API::Patch::PatchPostInstallHandler::doPatch_2_1_2',
      'API::Patch::PatchPostInstallHandler::doPatch_2_1_3',
      'API::Patch::PatchPostInstallHandler::doPatch_basic',
      'API::Patch::PatchPostInstallHandler::doPatch_checkPreCondition',
      'API::Patch::PatchPostInstallHandler::doPatch_universal',
      'API::Patch::PatchPostInstallHandler::toExecute',
      'API::Patch::PatchPostInstallHandler::useNow',
      'API::Patch::PatchPostInstallHandlerWebServer::callSubHandlerWithVersionSources',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_1_1_1',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_1_1_2',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_1_1_3',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_1_2_1',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_1_2_2',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_1_2_3',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_2_1_1',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_2_1_2',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_2_1_3',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_basic',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_checkPreCondition',
      'API::Patch::PatchPostInstallHandlerWebServer::doPatch_universal',
      'API::Patch::PatchPostInstallHandlerWebServer::toExecute',
      'API::Patch::PatchPostInstallHandlerWebServer::useNow',
      'API::Patch::PatchPostInstallHandlerBackup::callSubHandlerWithVersionSources',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_1_1_1',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_1_1_2',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_1_1_3',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_1_2_1',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_1_2_2',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_1_2_3',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_2_1_1',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_2_1_2',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_2_1_3',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_basic',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_checkPreCondition',
      'API::Patch::PatchPostInstallHandlerBackup::doPatch_universal',
      'API::Patch::PatchPostInstallHandlerBackup::toExecute',
      'API::Patch::PatchPostInstallHandlerBackup::useNow',
      'API::Patch::PatchPostInstallHandlerFiles::callSubHandlerWithVersionSources',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_1_1_1',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_1_1_2',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_1_1_3',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_1_2_1',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_1_2_2',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_1_2_3',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_2_1_1',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_2_1_2',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_2_1_3',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_basic',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_checkPreCondition',
      'API::Patch::PatchPostInstallHandlerFiles::doPatch_universal',
      'API::Patch::PatchPostInstallHandlerFiles::toExecute',
      'API::Patch::PatchPostInstallHandlerFiles::useNow'
    ];

    ok (Dumper($PatchSystemInfoHandler->validFunctions()) eq Dumper($Functions), 'functions correct');

    ok (not ($PatchSystemInfoHandler->isLastVersion('1.2.2')), 'version \'1.2.2\' is not last version');

    ok ($PatchSystemInfoHandler->isLastVersion('2.1.3'), 'version \'2.1.3\' is last version');

    ok (not ($PatchSystemInfoHandler->isFirstVersion('1.2.2')), 'version \'1.2.2\' is not first version');

    ok ($PatchSystemInfoHandler->isFirstVersion('1.1.1'), 'version \'1.1.1\' is first version');

    ok (($PatchSystemInfoHandler->firstVersion() eq '1.1.1'), 'first version eq \'1.1.1\'');

    ok (($PatchSystemInfoHandler->lastVersion() eq '2.1.3'), 'last version eq \'2.1.3\'');

    ok (($PatchSystemInfoHandler->nextVersion('1.2.3') eq '2.1.1'), 'next version from \'1.2.3\' is version \'2.1.1\'');

    eval{$PatchSystemInfoHandler->nextVersion('2.1.3')};
    ok( ExistsError(), 'next version from last version throws an error');

    ok (($PatchSystemInfoHandler->previousVersion('2.1.1') eq '1.2.3'), 'previous version from \'2.1.1\' is version \'1.2.3\'');

    eval{$PatchSystemInfoHandler->previousVersion('1.1.1')};
    ok( ExistsError(), 'previous version from first version throws an error');

    #TO DO validGroundFunction()

    ok (not ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not started for version \'2.1.1\'');

    $PatchSystemInfoHandler->registerStartedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');

    ok ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch'), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' now started for version \'2.1.1\'');

    ok (not ($PatchSystemInfoHandler->isPatchFunctionCompleted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not completed for version \'2.1.1\'');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');

    ok ($PatchSystemInfoHandler->isPatchFunctionCompleted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch'), 'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' now completed for version \'2.1.1\'');

    ok ($PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir') == undef, 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has no current version');

    $PatchSystemInfoHandler->setCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', '2.1.1');

    ok ($PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir') eq '2.1.1', 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has now current version \'2.1.1\'');

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.3', '2.1.2'), 'version \'1.1.3\' is lower than version \'2.1.2\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('2.1.2', '1.1.3')), 'version \'2.1.2\' is not lower than version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->versionLowerThan('6.6.6', '1.1.3')};
    ok( ExistsError(), 'check invalid number throws error');

    eval{$PatchSystemInfoHandler->versionLowerThan('1.1.3', '6.6.6')};
    ok( ExistsError(), 'check invalid number throws error');

    ok ($PatchSystemInfoHandler->versionGreaterThan('2.1.2', '1.1.3'), 'version \'2.1.2\' is greater than version \'1.1.3\'');

    ok (not ($PatchSystemInfoHandler->versionGreaterThan('1.1.3', '2.1.2')), 'version \'1.1.3\' is not greater than version \'2.1.2\'');

    eval{$PatchSystemInfoHandler->versionGreaterThan('6.6.6', '1.1.3')};
    ok( ExistsError(), 'check invalid number throws error');

    eval{$PatchSystemInfoHandler->versionGreaterThan('1.1.3', '6.6.6')};
    ok( ExistsError(), 'check invalid number throws error');


    #tests for cartridges version folder
    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does not exist for version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3')};
    ok( ExistsError(), 'get version dir without initialization throws error');

    $PatchSystemInfoHandler->setBasicAttribute('EpagesCartridgesVersionDir','C:\epages\Cartridges');

    ok ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3') eq 'C:/epages/Cartridges/1.1.3', 'cartridges version folder for version \'1.1.3\' correct \'C:/epages/Cartridges/1.1.3\'');

    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does not exist for version \'1.1.3\' without set manualy');

    $PatchSystemInfoHandler->setCartridgesVersionFolder('1.1.3', 'D:/elsewhere/Test/1.1.3');

    ok (($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does now exists exist for version \'1.1.3\'');

    ok ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3') eq 'D:/elsewhere/Test/1.1.3', 'cartridges version folder for version \'1.1.3\' correct changed \'D:/elsewhere/Test/1.1.3\'');

    ok ( $PatchSystemInfoHandler->getCartridgesVersionFolder('2.1.1') eq 'C:/epages/Cartridges/2.1.1', 'cartridges version folder for version \'2.1.1\' stay correct by default \'C:/epages/Cartridges/2.1.1\'');

    $PatchSystemInfoHandler->setCartridgesVersionFolder('1.1.3', 'C:/epages/Cartridges/1.1.3');

    ok ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3') eq 'C:/epages/Cartridges/1.1.3', 'cartridges version folder for version \'1.1.3\' correct setting back to \'C:/epages/Cartridges/1.1.3\'');


    #tests for patch version folder
    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','PatchVersionFolder')), 'version attribute \'PatchVersionFolder\' does not exist for version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->getPatchVersionFolder('1.1.3')};
    ok( ExistsError(), 'get version dir without initialization throws error');

    $PatchSystemInfoHandler->setBasicAttribute('PatchVersionDir','C:\Temp\patch_x_y_z\Cartridges.v');

    ok ( $PatchSystemInfoHandler->getPatchVersionFolder('1.1.3') eq 'C:/Temp/patch_x_y_z/Cartridges.v/1.1.3', 'patch version folder for version \'1.1.3\' correct \'C:/Temp/patch_x_y_z/Cartridges.v/1.1.3\'');

    ok ( $PatchSystemInfoHandler->getPatchVersionFolder('2.1.1') eq 'C:/Temp/patch_x_y_z/Cartridges.v/2.1.1', 'patch version folder for version \'2.1.1\' correct by default \'C:/Temp/patch_x_y_z/Cartridges.v/2.1.1\'');


    #test storetypes
    ok (not ($PatchSystemInfoHandler->existsStoreType('StoreType1')), 'store type \'StoreType1\' does not exist');

    $PatchSystemInfoHandler->addStoreType('StoreType1');

    ok ($PatchSystemInfoHandler->existsStoreType('StoreType1'), 'store type \'StoreType1\' now exist');

    $PatchSystemInfoHandler->addStoreType('StoreType2');
    $PatchSystemInfoHandler->addStoreType('StoreType6');
    $PatchSystemInfoHandler->addStoreType('StoreType7');
    $PatchSystemInfoHandler->addStoreType('StoreType5');
    $PatchSystemInfoHandler->addStoreType('StoreType3');
    $PatchSystemInfoHandler->addStoreType('StoreType4');
    $PatchSystemInfoHandler->addStoreType('AAA');

    my $aStoreTypes = [
      'AAA',
      'StoreType1',
      'StoreType2',
      'StoreType3',
      'StoreType4',
      'StoreType5',
      'StoreType6',
      'StoreType7'
    ];

    ok(Dumper($PatchSystemInfoHandler->getStoreTypes()) eq Dumper($aStoreTypes), 'store types correct');

    ok(not($PatchSystemInfoHandler->existsStoreTypeCartridge('StoreType1','CartridgeXY')), 'cartridge \'CartridgeXY\' dont exists in store type \'StoreType1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXY');

    ok($PatchSystemInfoHandler->existsStoreTypeCartridge('StoreType1','CartridgeXY'), 'cartridge \'CartridgeXY\' now exists in store type \'StoreType1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXB');
    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXG');
    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXH');
    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXZ');
    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXA');
    $PatchSystemInfoHandler->addStoreTypeCartridge('StoreType1', 'CartridgeXK');

    my $aCartridges = [
      'CartridgeXA',
      'CartridgeXB',
      'CartridgeXG',
      'CartridgeXH',
      'CartridgeXK',
      'CartridgeXY',
      'CartridgeXZ'
    ];

    ok(Dumper($PatchSystemInfoHandler->getStoreTypeCartridges('StoreType1')) eq Dumper($aCartridges), 'store type cartridges correct for store type \'StoreType1\'');


    #started patch function
    ok(scalar @{$PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1')} == 0, 'no strted patch function for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_1');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::ANYTHING::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid handler throws error');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_ANYTHING')};
    ok( ExistsError(), 'register patch funtion with invalid ground function name throws error');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('9.9.9', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid version throws error');

    my $aStartedPatchFunctions = [
        'API::Patch::PatchInstallHandlerCartridges::doPatch',
        'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerCartridges::doPatch_basic',
        'API::Patch::PatchInstallHandlerCartridges::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal'
    ];

    ok(Dumper($PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1')) eq Dumper($aStartedPatchFunctions), 'started patch functions correct for version \'1.1.1\'');


    #completed patch functions
    ok(scalar @{$PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1')} == 0, 'no completed patch function for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_1');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::ANYTHING::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid handler throws error');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_ANYTHING')};
    ok( ExistsError(), 'register patch funtion with invalid ground function name throws error');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('9.9.9', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch')};
    ok( ExistsError(), 'register patch funtion with invalid version throws error');

    my $aCompletedPatchFunctions = [
        'API::Patch::PatchInstallHandlerCartridges::doPatch',
        'API::Patch::PatchInstallHandlerCartridges::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerCartridges::doPatch_basic',
        'API::Patch::PatchInstallHandlerCartridges::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal'
    ];

    ok(Dumper($PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1')) eq Dumper($aCompletedPatchFunctions), 'completed patch functions correct for version \'1.1.1\'');

    #log level tests
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch') == 4, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('callSubHandlerWithVersionSources') == 3, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('toExecute') == 2, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_checkPreCondition') == 2, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_basic') == 1, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_universal') == 0, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_2_1_3') == 0, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('useNow') == 0, 'correct loglevel');

    #start time
    ok($StartTime eq $PatchSystemInfoHandler->getStartTime(), 'correct start time');

    #skip
    ok(not ($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler')), "correct skip value");

    $PatchSystemInfoHandler->setHandlerSkip('API::Patch::PatchPrepareHandler', 1);

    ok($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler'), "correct skip value");

    $PatchSystemInfoHandler->setHandlerSkip('API::Patch::PatchPrepareHandler', 0);

    ok(not ($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler')), "correct skip value");

}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [flags]

flags:
    -help          show the command line options

Example:
    perl $0
END_USAGE
    exit 2;
}

RunScript( Message => 'long', Sub => \&Main);

1;
